/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package rootgui.controller;

import java.io.IOException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;
import java.util.concurrent.BlockingQueue;
import java.util.concurrent.LinkedBlockingQueue;

import rootgui.model.Parameter;
import rootgui.model.SimulationRun;
import rootgui.model.TabPropertyChangeEvent;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import rootgui.model.recordstore.RecordStoreActionListener;
import rootgui.view.IConsoleOutput;
import simserver.AbsSimServerClient;
import simserver.SimServerClient;
import simserver.SimServerHTTPClient;

/**
 *
 * @author T. Ghetiu
 */
public class VRootController extends AbstractController {

//	private static String SERVER_NAME = "128.243.111.12"; // Teo
//	private static String SERVER_NAME = "128.243.111.15"; // John
	private static String SERVER_NAME = "http://simuplant.org"; // John
	private static int SERVER_PORT = 80;

	// Properties this controller expects to be stored in one or more registered
	// models
	private Map<Integer, AbsSimServerClient> sscConnections;
	/*
	 * User settings
	 */
	private Map<String, String> userSettings;

	/*
	 * SimServerClient
	 */
	// private simserver.AbsSimServerClient ssc;

	/*
	 * The fully qualified path to the tissue file
	 */
	public static final String TISSUE_FILE_NAME = "TissueFileName";

	/*
	 * The fully qualified path to the tissue file
	 */
	public static final String PARAMETER_SET = "ParameterSet";

	public static final String CURRENT_SIMULATION_RUN_CHANGED = "SimulationRunChanged";

	/*
	 * 
	 */
	public static final String MODEL_LOADED = "ModelLoaded";

	public static final String MODEL_UNLOADED = "ModelUnloaded";

	public static final String RECORD_UPDATED = "RecordUpdated";

	public static final String AVAILABLE_SIMULATION_MODEL = "AvailableSimulationModel";

	public static final String AVAILABLE_TISSUE_FILE = "AvailableTissueFile";

	public static final String AVAILABLE_SIMULATION_MODELS_CHANGED = "AvailableSimulationModelsChanged";

	public static final String MAX_ITERATIONS = "MaxIterations";

	public static final String INC_ITERATION = "IncIteration";

	public static final String ITERATION_UPDATED = "IterationUpdated";

	public static final String ITERATION = "Iteration";

	public static final String SIMULATION_INITIALISED = "SimulationInitialised";

	public static final String SIMULATION_IN_PROGRESS = "SimInProgress";

	public static final String TIMESTEP = "Timestep";

	public static final String MODEL_SPECIES_NAMES = "ModelSpeciesNames";

	public static final String TISSUE_FILE_INITIALISED = "TissueFileInitialised";

	public static final String SSC_CONNECTED = "SSCConnected";

	public static final String SIMULATION_RUN_CREATED = "SRCreated";

	public static final String SIMULATION_RUN_REMOVED = "SRRemoved";

	public VRootController() {
		rbql = new RunnableBlockingQueueList("VRC");
		sscConnections = new HashMap<Integer, AbsSimServerClient>();
		loadUserSettings();
	}

	/*
	 * Returns user saved settings
	 */
	public Map<String, String> getUserSettings() {
		return userSettings;
	}

	/*
	 * Loads user settings
	 */
	private void loadUserSettings() {
		userSettings = new HashMap<String, String>();
	}

	/*
	 * Saves user settings, for the next session.
	 */
	private void saveUserSettings() {

	}

	@Override
	public void setSimulationRun(int simID, SimulationRun simRun) {
		try {
			AbsSimServerClient ssc = sscConnections.get(simID);
//			ssc.setTissueFile(simRun.getTissueFileName());
//			ssc.initTissue();
			Record r = simRun.getRecord();
			for (Species s : r.getSpecies()) {
				ssc.setSpeciesData(s.getName(), r.getSpeciesData(s));
			}
			ssc.setTimestep(simRun.getTimestep());

			for (SimulationModel sm : simRun.getLoadedSimulationModels().values()){
				ssc.addEditedModel(simRun.getAvailableSimulationModel(sm.getName()), sm);
			}
			
			simRun.addRecordStoreActionListener(this);
			// setModelProperty(SSC_CONNECTED, simID, true);
		} catch (Exception e) {
			// setModelProperty(SSC_CONNECTED, simID, false);
			e.printStackTrace();
		}
		model.setSimulationRun(simID, simRun);
	}

	@Override
	public SimulationRun getSimulationRun(int simID) {
		return model.getSimulationRun(simID);
	}

	@Override
	public void createSimulationRun(final int simID) {
		try {

			// creating server connection
			AbsSimServerClient ssc;
			consO.printlnStd(simID, "Establishing server connection...");
			ssc = new SimServerHTTPClient(simID, SERVER_NAME, SERVER_PORT, null);// consO
			if (sscConnections.containsKey(simID))
				System.err.println("VRootController.createSimulationRun(" + simID + ") error, non-unique simID.");
			sscConnections.put(simID, ssc);
			setModelProperty(SSC_CONNECTED, simID, true);

			rbql.createQueue(simID);

			// creating SimulationRun object
			model.createSimulationRun(simID, this);
			loadAvailableModels(simID);
			loadAvailableTissueNames(simID);
		} catch (Exception e) {
			setModelProperty(SSC_CONNECTED, simID, false);
			// consO.printErr(simID, "Server connection failed...");
			// e.printStackTrace();
		}
	}

	public void setModel(final int simID, final String modelName) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "setModel(" + simID + ", " + modelName + ")";
			}

			@Override
			public void run() {
				try {
					sscConnections.get(simID).addModelByName(modelName);
					SimulationModel sm = sscConnections.get(simID)
							.getSimulationModel(modelName);
					setModelProperty(MODEL_LOADED, simID, sm);
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	private Runnable getStepRunnable(final int simID) {
		Runnable rb = new Runnable() {
			@Override
			public String toString() {
				return "step()";
			}

			@Override
			public void run() {
				Record r = null;
				if (model.isExecutionAllowed(simID)) {

					// setModelProperty(SIMULATION_IN_PROGRESS, simID, true);

					AbsSimServerClient ssc = sscConnections.get(simID);
					ssc.step();

					// setModelProperty(SIMULATION_IN_PROGRESS, simID, false);
					r = getRecord(simID);
					if (r != null)
						r = ssc.getRecord(r.getGeometry(), null);
					else r = null;
					setModelProperty(INC_ITERATION, simID, r);
					
				} else {
					consO.printlnStd(simID, "Warning: Execution not allowed!");
				}
			}
		};
		return rb;
	}

	@Override
	public void stepNonBlocking(final int simID) {
		rbql.addRunnableToQueue(simID, getStepRunnable(simID));
	}

	@Override
	public void step(final int simID) {
		rbql.addBlockingRunnableToQueue(simID, getStepRunnable(simID));
	}

	@Override
	public void cancel(int simID) {
		// TODO Auto-generated method stub

	}

	@Override
	public void setTissueFileName(final int simID, final String fileName) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "setTissueFileName(" + simID + ", " + fileName + ")";
			}

			@Override
			public void run() {
				model.getSimulationRun(simID).clearRecordStore();
				model.getSimulationRun(simID).setTissueFileInitialized(false);
				sscConnections.get(simID).setTissueFile(fileName);
				setModelProperty(TISSUE_FILE_NAME, simID, fileName);
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	@Override
	public String getTissueFileName(int simID) {
		return model.getTissueFileName(simID);
	}

	@Override
	public void initTissue(final int simID) {
		Runnable r = new Runnable() {

			@Override
			public String toString() {
				return "initTissue(" + simID + ")";
			}

			@Override
			public void run() {
				AbsSimServerClient ssc = sscConnections.get(simID);
				if (model.getSimulationRun(simID).isTissueFileInitialized())
					return;
				
				if (!ssc.initTissue()) {
					setModelProperty(TISSUE_FILE_INITIALISED, simID, new Record(null));
				} else {
					// load a first record
					Record firstRecord = null;
					if (getRecord(simID) != null)
						firstRecord = ssc.getRecord(getRecord(simID).getGeometry(), null);
					else
						firstRecord = ssc.getRecord();
					model.setInitialRecord(simID, firstRecord);
					setModelProperty(TISSUE_FILE_INITIALISED, simID, firstRecord);
				}
			}
		};
		rbql.addRunnableToQueue(simID, r);
		// rbql.blockTillQueueEmpty(simID);
	}

	@Override
	public void setModelParameter(final int simID, final String modelName,
			final String parameter, final String value) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "setModelParameter(" + simID + ", " + modelName + ", "
						+ parameter + ", " + value + ")";
			}

			@Override
			public void run() {
				sscConnections.get(simID).setParameter(modelName, parameter, value);
				model.getSimulationRun(simID).getLoadedSimulationModel(modelName).setParameter(parameter, value);
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	@Override
	public void setMaxIterations(final int simID, final int iterations) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "setMaxIterations(" + simID + ", " + iterations + ")";
			}

			@Override
			public void run() {
				sscConnections.get(simID).setIterations(iterations);
				setModelProperty(MAX_ITERATIONS, simID, iterations);
				// model.getSimulationRun(simID).setIterations(iterations);
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	@Override
	public void setTimestep(final int simID, final double timestep) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "setTimestep()";
			}

			@Override
			public void run() {
				sscConnections.get(simID).setTimestep(timestep);
				setModelProperty(TIMESTEP, simID, timestep);
				// model.getSimulationRun(simID).setTimestep(timestep);
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}
	
	@Override
	public void setGrowth(final int simID, final boolean onOff) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "setGrowth()";
			}

			@Override
			public void run() {
				sscConnections.get(simID).setGrowth(onOff);
				
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

//	@Override
//	public String getSpeciesPublicName(int simID, Species species) {
//
//			if (model.hasSimID(simID))
//				return sscConnections.get(simID).getSpeciesPublicName(species);
//
//		return null;
//	}
	
	@Override
	public boolean isConnected(int simID) {
		return sscConnections.get(simID).isConnected();
	}

	@Override
	public void disconnectSimulation(int simID) {
		sscConnections.get(simID).disconnect();
	}

	@Override
	public void resetSimulation(final int simID){
		sscConnections.get(simID).reset();

		setIteration(simID, 0);
		SimulationRun simr =  model.getSimulationRun(simID);
		simr.setTissueFileInitialized(false);
		simr.getRecordStore().clear();
	}
	
	@Override
	public void setIteration(final int simID, final int iteration) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "setIteration(" + simID + ", " + iteration + ")";
			}

			@Override
			public void run() {
				sscConnections.get(simID).setCurrentIteration(iteration);
				setModelProperty(ITERATION, simID, iteration);
			}
		};
		rbql.addBlockingRunnableToQueue(simID, r);
	}

	@Override
	public void updateData(final int simID, final Species targetSpecies) {
		Runnable rb = new Runnable() {
			@Override
			public String toString() {
				return "updateData(" + simID + ")";
			}

			@Override
			public void run() {
				Record r = getRecord(simID);
				if (r != null)
					r = sscConnections.get(simID).getRecord(r.getGeometry(),
							targetSpecies);
				else
					r = sscConnections.get(simID).getRecord(null,
							targetSpecies);
				setModelProperty(ITERATION_UPDATED, simID, r);
			}
		};
		rbql.addRunnableToQueue(simID, rb);
	}

	@Override
	public void updateData(int simID) {
		updateData(simID, null);
	}

	public void loadSimulationModel(final int simID, final String modelName) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "loadSimulationModel(" + simID + ", " + modelName
						+ ")";
			}

			@Override
			public void run() {
				try {
					AbsSimServerClient ssc = sscConnections.get(simID);
					SimulationModel sm = ssc.getSimulationModel(modelName);
					ssc.addModel(sm); // addEditedModel
					setModelProperty(MODEL_LOADED, simID, sm);
					// model.getSimulationRun(simID).getLoadedSimulationModels()
				} catch (Exception e) {
					e.printStackTrace();
				}
			}
		};
		rbql.addRunnableToQueue(simID, r);
		// rbql.blockTillQueueEmpty(simID);
	}

	@Override
	public void loadSimulationModel(final int simID, final SimulationModel sm) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "loadSimulationModel()";
			}

			@Override
			public void run() {
				sscConnections.get(simID).addEditedModel(model.getSimulationRun(simID).getAvailableSimulationModel(sm.getName()), sm); // addEditedModel
				setModelProperty(MODEL_LOADED, simID, sm);
				// model.getSimulationRun(simID).getLoadedSimulationModels()
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	@Override
	public void unloadSimulationModel(int simID, String modelName) {
		// TODO model.getSimulationRun(simID).unlo
	}

	@Override
	public void setCellCarrier(final int simID, final String[] groups,
			final boolean[] onOff) {
		// rbql.blockTillQueueEmpty(simID);
		Runnable r = new Runnable() {

			@Override
			public void run() {
				// TODO Auto-generated method stub
				AbsSimServerClient ssc = sscConnections.get(simID);
				boolean b = ssc.setCellCarrier(groups, onOff);
				// setModelProperty(RECORD_UPDATED, simID, ssc.getRecord());
				updateData(simID);
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	@Override
	public Record getRecord(int simID, int iteration) {
		if (model.hasSimID(simID))
			return model.getSimulationRun(simID).getRecord(iteration);
		return null;
	}

	@Override
	public String[] getAvailableModelNames(int simID) {
		try {
			if (model.hasSimID(simID))
				return sscConnections.get(simID).getAvailableModelsNames();
		} catch (IOException e) {
		}
		return null;
	}

	@Override
	public int getIteration(int simID) {
		if (model.hasSimID(simID))
			return model.getSimulationRun(simID).getIteration();
		return -1;
	}

	@Override
	public int getMaxIterations(int simID) {
		if (model.hasSimID(simID))
			return model.getSimulationRun(simID).getMaxIterations();
		return -1;
	}

	@Override
	public Map<String, Parameter> getModelDefaultParameters(int simID,
			String modelName) {
		return getAvailableSimulationModel(simID, modelName).getParameters();
	}

	@Override
	public List<Species> getModelSpecies(int simID, String modelName) {
		return model.getSimulationRun(simID).getModelSpecies(modelName);
	}

	@Override
	public SimulationModel getAvailableSimulationModel(int simID,
			String modelName) {
		return model.getSimulationRun(simID).getAvailableSimulationModel(
				modelName);
	}

	@Override
	public SimulationModel getLoadedSimulationModel(int simID,
			String modelName) {
		return model.getSimulationRun(simID).getLoadedSimulationModel(
				modelName);
	}

	@Override
	public String[] getLoadedSimulationModelNames(int simID) {
		return null;
	}

	@Override
	public Map<String, Species> getLoadedSpecies(int simID) {
		if (model.getSimulationRun(simID).getIteration() > 1)
			return model.getSimulationRun(simID).getLoadedSpecies();
		Map<String, Species> map = new HashMap<String, Species>();
		Set<Species> set = sscConnections.get(simID).getLoadedSpecies();
		for (Species s : set) {
			map.put(s.getName(), s);
		}
		return map;
	}

	@Override
	public Species[] getLoadedSpeciesArray(final int simID) {
		// Species[] result = null;
		// if (model.getSimulationRun(simID).getIteration() > 0)
		// result = model.getSimulationRun(simID).getLoadedSpeciesArray();
		// if (result != null && result.length > 0)
		// return result;
		return sscConnections.get(simID).getLoadedSpecies().toArray(new Species[] {});
	}

	@Override
	public Parameter getLoadedParameter(int simID, String modelName,
			String parameter) {
		return model.getSimulationRun(simID).getLoadedParameter(modelName,
				parameter);
	}

	@Override
	public Record getLastAvailableRecord(int simID) {
		return model.getSimulationRun(simID).getLastAvailableRecord();
	}

	@Override
	public Record getRecord(int simID) {
		return model.getSimulationRun(simID).getRecord();
	}

	public void loadAvailableTissueNames(final int simID) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "loadAvailableTissueNames()";
			}

			@Override
			public void run() {
		try {
			String[] tissueNames = sscConnections.get(simID).getAvailableTissueNames();
			
			for (String tissueName : tissueNames) {						
				setModelProperty(AVAILABLE_TISSUE_FILE, simID, tissueName);
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	/*
	 * Loads into the model the available simulation models provided by the
	 * SimServerClient
	 */
	public void loadAvailableModels(final int simID) {
		Runnable r = new Runnable() {
			@Override
			public String toString() {
				return "loadAvailableModels()";
			}

			@Override
			public void run() {
				try {
					String[] modelNames = sscConnections.get(simID).getAvailableModelsNames();

					for (String modelName : modelNames) {
						SimulationModel sm = sscConnections.get(simID).getSimulationModel(modelName);
						setModelProperty(AVAILABLE_SIMULATION_MODEL, simID, sm);
					}
				} catch (Exception e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	@Override
	public void setSimulationInitialised(final int simID) {
		Runnable r = new Runnable() {

			@Override
			public void run() {
				if (sscConnections.get(simID).initSimulation())
					setModelProperty(SIMULATION_INITIALISED, simID, true);
				
			}
		};
		rbql.addRunnableToQueue(simID, r);
		updateData(simID);
	}

	@Override
	public double getTimestep(int simID) {
		return model.getTimestep(simID);
	}

	@Override
	public int getSimulationRunCount() {
		return model.getSimulationRunCount();
	}

	@Override
	public void removeSimulationRun(final int simID) {
		rbql.removeQueue(simID);
		sscConnections.get(simID).disconnect();
		sscConnections.remove(simID);

		model.removeSimulationRun(simID);
	}

	@Override
	public void removeAllSimulationRuns() {
		rbql.removeAllQueues();
		for (int simID : sscConnections.keySet()) {
			sscConnections.get(simID).disconnect();
			sscConnections.remove(simID);

			model.removeSimulationRun(simID);
		}
	}

	@Override
	public int getRecordCount(int simID) {
		return model.getRecordCount(simID);
	}

	/* RECORD STORE ACTION LISTENER */

	@Override
	public void fireRecordAdded(int iteration) {
		// TODO Auto-generated method stub

	}

	@Override
	public void fireRecordDeleted(int iteration) {
		// TODO Auto-generated method stub

	}

	@Override
	public void fireRecordUpdated(int iteration) {
		// TODO Auto-generated method stub
//		System.err.println("vrootcontrolller: fireRecordUpdated(" + iteration + ")");
	}

	@Override
	public void fireCellSpeciesUpdated(Record r, String cellID, String speciesName, double value) {
		// TODO Auto-generated method stub
		// This should be getSimID I think, not getIndex.
		int simID = r.getStore().getSimID();
			try {			
			if (sscConnections.containsKey(simID)) {
				sscConnections.get(simID).setCellSpeciesValue(cellID, speciesName, value);
				propertyChange(new TabPropertyChangeEvent(simID, this,
					RECORD_UPDATED, null, r.getIteration()));
			} else {
				System.err.println("VRootController.fireCellSpeciesUpdated error: sscConnections.containsKey(" + simID + ") == false");
			}
		} catch (Exception e) {
			// TODO Auto-generated catch block
//			e.printStackTrace();
		}
	}

	public String getServerName() {
		return SERVER_NAME;
	}

	public void setServerName(String host) {
		SERVER_NAME = host;
	}

	public int getServerPort() {
		return SERVER_PORT;
	}

	public void setServerPort(int port) {
		SERVER_PORT = port;
	}

	@Override
	public void newSimulation(final int simID) {
		final RecordStoreActionListener rsal = this;
		Runnable r = new Runnable() {
			
			@Override
			public void run() {
				try {
					sscConnections.get(simID).reconnect();
				} catch (UnknownHostException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				} catch (IOException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				model.newSimulationRun(simID, rsal);
			}
		};
		rbql.addRunnableToQueue(simID, r);
	}

	@Override
	public boolean hasSimID(int simID) {
		return sscConnections.containsKey(simID);
	}
}
