package rootgui.model;

import java.util.List;
import java.util.Map;

import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;

public interface IDataProvider {
	/*
	 * Returns a String[] of models available in the simulation server
	 */
	public String[] getAvailableModelNames();

	/*
	 * Returns the current iteration of the simulation
	 */
	public int getIteration();

	/*
	 * Returns the number of iterations the simulation should run for
	 */
	public int getMaxIterations();

	/*
	 * Returns a model's default parameters
	 */
	public Map<String, Parameter> getModelDefaultParameters(String modelName);

	/*
	 * Returns the species of a model
	 */
	public List<Species> getModelSpecies(String modelName);

	/*
	 * Returns a (default) SimulationModel by name
	 */
	public SimulationModel getAvailableSimulationModel(String modelName);

	/*
	 * Returns a (loaded) SimulationModel by name
	 */
	public SimulationModel getLoadedSimulationModel(String modelName);

	public String[] getLoadedSimulationModelNames();

	/*
	 * Returns a map of species names (the keys) and Species types (the values) 
	 */
	public Map<String, Species> getLoadedSpecies();
	
	/*
	 * Returns a String[] of species that are available (loaded) in the simulation server
	 */
	public Species[] getLoadedSpeciesArray();

	/*
	 * Returns a Parameter value from the simulation 
	 */
	public Parameter getLoadedParameter(String modelName, String parameter);

	/*
	 * 
	 */
	public Record getLastAvailableRecord();
	
	/*
	 * Returns the Record of the current iteration
	 */
	public Record getRecord();

	/*
	 * Returns the number of records in the data record store
	 */
	// public int getRecordCount();

	public String getTissueFileName();
	
	public SimulationRun getSimulationRun();
}
