package rootgui.model;

import java.util.List;
import java.util.Map;

import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;

/*
 * Interface making an implementing class, a simulation data provider.
 */
public interface ITabDataProvider {
	/*
	 * Returns a String[] of models available in the simulation server
	 */
	public String[] getAvailableModelNames(int simID);

	/*
	 * Returns the current iteration of the simulation
	 */
	public int getIteration(int simID);

	/*
	 * Returns the number of iterations the simulation should run for
	 */
	public int getMaxIterations(int simID);
	
	public double getTimestep(int simID);

	/*
	 * Returns a model's default parameters
	 */
	public Map<String, Parameter> getModelDefaultParameters(int simID, String modelName);

	/*
	 * Returns the species of a model
	 */
	public List<Species> getModelSpecies(int simID, String modelName);

	/*
	 * Returns a (default) SimulationModel by name
	 */
	public SimulationModel getAvailableSimulationModel(int simID, String modelName);

	/*
	 * Returns a (loaded) SimulationModel by name
	 */
	public SimulationModel getLoadedSimulationModel(int simID, String modelName);

	/*
	 * Returns a String[] of model names that have been loaded in a simulation
	 */
	public String[] getLoadedSimulationModelNames(int simID);

	/*
	 * Returns a map of species names (the keys) and Species types (the values)
	 */
	public Map<String, Species> getLoadedSpecies(int simID);

	/*
	 * Returns a String[] of species that are available (loaded) in the
	 * simulation server
	 */
	public Species[] getLoadedSpeciesArray(int simID);

	/*
	 * Returns a Parameter value from the simulation
	 */
	public Parameter getLoadedParameter(int simID, String modelName, String parameter);

	/*
	 * 
	 */
	public Record getLastAvailableRecord(int simID);

	/*
	 * Returns the Record of the current iteration
	 */
	public Record getRecord(int simID);
	
	public SimulationRun getSimulationRun(final int simID);
}
