/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package rootgui.model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import rootgui.controller.VRootController;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.IRecordStore;
import rootgui.model.recordstore.Record;
import rootgui.model.recordstore.RecordStore;
import rootgui.model.recordstore.RecordStoreActionListener;
import rootgui.model.recordstore.SpeciesRecord;
import visualiser.viewpanel.controller.CDPController;

/**
 * 
 * @author tghetiu
 */
public class VRootModel extends AbstractModel {

	/*
	 * 
	 */
	// protected String tissueFileName;

	/*
	 * Map where keys are model names and values are maps of K = parameter
	 * names, V = Parameter
	 */
	// protected Map<String, SimulationModel> availableSimulationModels;

	public VRootModel() {
		// availableSimulationModels = new HashMap<String, SimulationModel>();
		simulationRuns = new HashMap<Integer, SimulationRun>();
		// createSimulationRun();
		//
		// setSimInProgress(0, false);
	}

	// Accessors

	/*
	 * Creates another SimulationRun record and sets the active simulation
	 * number to its index
	 */
	@Override
	public void createSimulationRun(final int simID, RecordStoreActionListener rsal) {
		SimulationRun sr = new SimulationRun(rsal, simID);
		simulationRuns.put(simID, sr);
		firePropertyChange(simID, VRootController.SIMULATION_RUN_CREATED, null, true);
	}

	@Override
	public void removeSimulationRun(int simID) {
		simulationRuns.remove(simID);
		firePropertyChange(simID, VRootController.SIMULATION_RUN_REMOVED, null, true);
	}

	public void setIterationUpdated(int simID, Record r) {
		SimulationRun sr = simulationRuns.get(simID);
		Record updatedRecord = sr.setIterationUpdated(r);
		firePropertyChange(simID, VRootController.ITERATION_UPDATED, null, updatedRecord);
	}

	/*
	 * Increases the current iteration count and adds a data Record to the
	 * recordStore
	 * 
	 * @see rootgui.model.AbstractModel#incIteration()
	 */
	public int setIncIteration(int simID, Record r) {
		SimulationRun sr = simulationRuns.get(simID);
		sr.setIncIteration(r);
		firePropertyChange(simID, VRootController.INC_ITERATION, null, r);
		return sr.getIteration();
	}

	public int setInitialRecord(int simID, Record r) {
		SimulationRun sr = simulationRuns.get(simID);
		sr.setInitialRecord(r);
		// firePropertyChange(VRootController.INC_ITERATION, null, r);
		return sr.getIteration();
	}

	/*
	 * Removes data corresponding to all iterations starting from a given number
	 */
	@Override
	public int removeIterations(int simID, int startingIteration) {
		// TODO
		IRecordStore irs = simulationRuns.get(simID).getRecordStore();
		irs.removeRecordsStartingFrom(startingIteration);
		return irs.size();
	}

	/*
	 * Sets the maximum number of iterations to be run
	 */
	@Override
	public void setMaxIterations(int simID, Integer iterations) {
		simulationRuns.get(simID).setMaxIterations(iterations.intValue());
		firePropertyChange(simID, VRootController.MAX_ITERATIONS, "",
				iterations);
	}

	public void setIteration(int simID, Integer iteration) {
		simulationRuns.get(simID).setIteration(iteration);
		firePropertyChange(simID, VRootController.ITERATION, "", iteration);
	}

	public String[] getLoadedModelNames(int simID) {
		return simulationRuns.get(simID).getLoadedSimulationModels().keySet()
				.toArray(new String[0]);
	}

	// /*
	// * Loads a model
	// */
	// @Override
	// public void setModelLoaded(int simID, String modelName) throws
	// Exception {
	// verify modelName
	// SimulationModel sm = availableSimulationModels.get(modelName);
	// if (sm == null) {
	// throw new Exception(
	// "Model name does not correspond to any available models");
	// }
	// SimulationModel sm2 = (SimulationModel) sm.clone();
	// simulationRuns.get(simID).getLoadedSimulationModels().put(modelName,
	// sm2);
	// firePropertyChange(simID, VRootController.MODEL_LOADED, "", sm2);
	// }

	public void setSimulationInitialised(int simID, Boolean onOff) {
		simulationRuns.get(simID).setSimulationInitialized(onOff.booleanValue());
		firePropertyChange(simID, VRootController.SIMULATION_INITIALISED, null, onOff.booleanValue());
	}

	@Override
	public void setModelLoaded(int simID, SimulationModel sm)
			throws Exception {
		// verify modelName
		if (sm == null) {
			throw new Exception(
					"Model name does not correspond to any available models");
		}
		simulationRuns.get(simID).addLoadedSimulationModel(sm);
		firePropertyChange(simID, VRootController.MODEL_LOADED, null, sm);
	}

	@Override
	public void setModelUnloaded(int simID, String modelName) {
		// verify modelName
		if (simulationRuns.get(simID).getLoadedSimulationModels()
				.containsKey(modelName)) {
			simulationRuns.get(simID).removeLoadedSimulationModel(modelName);

			firePropertyChange(simID, VRootController.MODEL_UNLOADED, null,
					modelName);
		} else {
			System.out.println("VRootModel.setModelUnloaded: model <"
					+ modelName + "> was not loaded!");
		}
	}

	@Override
	public void setTimestep(int simID, Double timestep) {
		double oldTimestep = simulationRuns.get(simID).getTimestep();
		simulationRuns.get(simID).setTimestep(timestep);
		firePropertyChange(simID, VRootController.TIMESTEP, oldTimestep,
				timestep);
	}

	@Override
	public void setTissueFileName(int simID, String fileName) {
		String oldTissueFileName = simulationRuns.get(simID)
				.getTissueFileName();
		simulationRuns.get(simID).setTissueFileName(fileName);
		firePropertyChange(simID, VRootController.TISSUE_FILE_NAME,
				oldTissueFileName, fileName);
	}

	@Override
	public String getTissueFileName(int simID) {
		try {
			return simulationRuns.get(simID).getTissueFileName();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			return null;
		}
	}

	@Override
	public void setModelParameter(int simID, String modelName,
			String parameterName, String value) {
		try {
			// if (!simulationRuns.get(simID).getLoadedSimulationModels()
			// .containsKey(modelName)) {
			// simulationRuns
			// .get(simID).getLoadedSimulationModels().put(modelName,
			// availableSimulationModels.get(modelName).clone());
			// }
			SimulationModel sm = simulationRuns.get(simID)
					.getLoadedSimulationModels().get(modelName);
			Parameter p = new Parameter(parameterName, value);
			sm.getParameters().put(parameterName, p);

			firePropertyChange(simID, VRootController.PARAMETER_SET, "", p);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}

	}

	@Override
	public void setAvailableSimulationModel(int simID, SimulationModel sm) {
		if (simulationRuns.get(simID).getAvailableSimulationModel(
				sm.getName()) == null) {
			// TODO new simulation model available; maybe fire an event
		}
		simulationRuns.get(simID).addAvailableSimulationModel(sm);
		firePropertyChange(simID, VRootController.AVAILABLE_SIMULATION_MODEL,
				"", sm);
	}
	
	public void setAvailableTissueFile(int simID, String tissueName){
		firePropertyChange(simID, VRootController.AVAILABLE_TISSUE_FILE, "", tissueName);
	}

	@Override
	public void clearAvailableSimulationModels(int simID) {
		simulationRuns.get(simID).clearAvailableSimulationModels();
		firePropertyChange(simID,
				VRootController.AVAILABLE_SIMULATION_MODELS_CHANGED, "", null);
	}

	/* ACCESSORS */

	public SpeciesRecord getSpeciesRecord(int simID, Species species) {
		return getSpeciesRecord(simID, species,
				simulationRuns.get(simID).getIteration());
	}

	public SpeciesRecord getSpeciesRecord(int simID, Species species,
			int iteration) {
		return simulationRuns.get(simID).getSpeciesRecord(species,
				iteration);
	}

	@Override
	public void clear(int simID) {
		// TODO
	}

	// @Override
	// public Geometry getGeometry() {
	// // return recordsStore.getRecord(iteration).getGeometry();
	// return simulationRuns.get(simID).getGeometry();
	// }
	//
	// @Override
	// public void setGeometry(Geometry geometry) {
	// simulationRuns.get(simID).setGeometry(geometry);
	// }

	// @Override
	// public int getIteration() {
	// return simulationRuns.get(simID).getIteration();
	// }

	public int getMaxIterations(int simID) {
		return simulationRuns.get(simID).getMaxIterations();
	}

	// @Override
	// public Map<String, Parameter> getModelDefaultParameters(String modelName)
	// {
	// return availableSimulationModels.get(modelName).getParameters();
	// }
	//
	// @Override
	// public List<Species> getModelSpecies(String modelName) {
	// return availableSimulationModels.get(modelName).getSpecies();
	// }

	/*
	 * (non-Javadoc)
	 * 
	 * @see rootgui.model.IDataProvider#getLoadedSpecies()
	 */
	// @Override
	// public Map<String, Species> getLoadedSpecies() {
	// Map<String, Species> speciesMap = new HashMap<String, Species>();
	// for (String model : simulationRuns.get(simID)
	// .getLoadedSimulationModels().keySet()) {
	// SimulationModel sm = availableSimulationModels.get(model);
	// for (Species species : sm.getSpecies())
	// speciesMap.put(species.getName(), species);
	// }
	//
	// return speciesMap;
	// }

	// @Override
	// public Species[] getLoadedSpeciesArray() {
	// return getLoadedSpecies().values().toArray(new Species[] {});
	// }
	//
	// @Override
	// public Parameter getLoadedParameter(String modelName, String parameter) {
	// return availableSimulationModels.get(modelName).getParameters()
	// .get(parameter);
	// }
	//
	// @Override
	// public Record getRecord(int iteration) {
	// if (iteration < simulationRuns.get(simID).getRecordStore()
	// .size())
	// return simulationRuns.get(simID).getRecordStore()
	// .getRecord(iteration);
	// return null;
	// }

	@Override
	public Geometry getGeometry(int simID, int iteration) {
		return simulationRuns.get(simID).getRecordStore()
				.getRecord(iteration).getGeometry();
	}

	@Override
	public void readGeometryFromFile(int simID, String cellSetFile, int iteration) throws Exception {
		Record r = null;
		if (simulationRuns.get(simID).getRecordStore().size() == 0 || iteration == simulationRuns.get(simID).getRecordStore()
						.size()) {
			r = new Record(null);
			simulationRuns.get(simID).getRecordStore().addRecord(r);
		} else if (iteration < simulationRuns.get(simID).getRecordStore().size())
			r = simulationRuns.get(simID).getRecordStore().getRecord(iteration);
		else
			throw new Exception(
					"You can't set the geometry for more than one iteration away");
		r.setGeometryFromFile(cellSetFile);
	}

	@Override
	public void readSpeciesFromFile(int simID, Species species,
			String csvFile, int iteration) throws Exception {
		Record r = null;
		if (simulationRuns.get(simID).getRecordStore().size() == 0
				|| iteration == simulationRuns.get(simID).getRecordStore()
						.size()) {
			r = new Record(null);
			simulationRuns.get(simID).getRecordStore().addRecord(r);
		} else if (iteration < simulationRuns.get(simID).getRecordStore().size())
			r = simulationRuns.get(simID).getRecordStore().getRecord(iteration);
		else
			throw new Exception(
					"You can't set the a species record for more than one iteration away");
		r.setCellRecordFromCSV(species, csvFile);
	}

	// @Override
	// public Record getRecord() {
	// return getRecord(simulationRuns.get(simID).getIteration());
	// }
	//
	// @Override
	// public SimulationModel getAvailableSimulationModel(String modelName) {
	// return availableSimulationModels.get(modelName);
	// }
	//
	// @Override
	// public SimulationModel getLoadedSimulationModel(String modelName) {
	// return simulationRuns.get(simID)
	// .getLoadedSimulationModels().get(modelName);
	// }

	// @Override
	// public String[] getLoadedSimulationModelNames() {
	// return simulationRuns.get(simID)
	// .getLoadedSimulationModels().keySet().toArray(new String[] {});
	// }

	@Override
	public void clearRecordStore(int simID) {
		simulationRuns.get(simID).clearRecordStore();
	}

	public boolean isSimInProgress(int simID) {
		return simulationRuns.get(simID).isSimInProgress();
	}

	public void setSimInProgress(int simID, Boolean simInProgress) {
		boolean oldValue = simulationRuns.get(simID).isSimInProgress();
		simulationRuns.get(simID).setSimInProgress(simInProgress);
		firePropertyChange(simID, VRootController.SIMULATION_IN_PROGRESS,
				oldValue, simInProgress);

	}

	public void setSSCConnected(int simID, Boolean onOff) {
		firePropertyChange(simID, VRootController.SSC_CONNECTED, null, onOff);
	}

	/*
	 * Returns the last avaiable record where there is a non null Geometry
	 * 
	 * @see rootgui.model.IDataProvider#getLastAvailableRecord()
	 */
	// @Override
	// public Record getLastAvailableRecord() {
	// IRecordStore irs = simulationRuns.get(simID)
	// .getRecordStore();
	// for (int i = irs.size() - 1; i >= 0; i--) {
	// Record r = irs.getRecord(i);
	// if (r.getGeometry() != null)
	// return r;
	// }
	// return null;
	// }

	/* RECORD STORE ACTION LISTENER */

	// @Override
	// public void fireRecordUpdated(int iteration) {
	// firePropertyChange(VRootController.RECORD_UPDATED, null, iteration);
	// }

	public void setTissueFileInitialised(int simID, Record r) {
		simulationRuns.get(simID).setTissueFileInitialized(
				r.getIteration() != Record.NO_ITERATION);
		firePropertyChange(simID, VRootController.TISSUE_FILE_INITIALISED,
				false, r);
	}

	public boolean isTissueFileInitialised(int simID) {
		return simulationRuns.get(simID).isTissueFileInitialized();
	}

	public boolean isExecutionAllowed(int simID) {
		return isTissueFileInitialised(simID)
				&& (getLoadedSimulationModelNames(simID).length > 0);
	}

	@Override
	public Record getRecord(int simID, int iteration) {
		try {
			return simulationRuns.get(simID).getRecord(iteration);
		} catch (Exception e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			return null;
		}
	}

	@Override
	public String[] getAvailableModelNames(int simID) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public int getIteration(int simID) {
		return simulationRuns.get(simID).getIteration();
	}

	@Override
	public Map<String, Parameter> getModelDefaultParameters(int simID,
			String modelName) {
		return getAvailableSimulationModel(simID, modelName).getParameters();
	}

	@Override
	public List<Species> getModelSpecies(int simID, String modelName) {
		return simulationRuns.get(simID).getModelSpecies(modelName);
	}

	@Override
	public SimulationModel getAvailableSimulationModel(int simID,
			String modelName) {
		// TODO Auto-generated method stub
		return null;
	}

	@Override
	public SimulationModel getLoadedSimulationModel(int simID,
			String modelName) {
		return simulationRuns.get(simID).getLoadedSimulationModel(modelName);
	}

	@Override
	public String[] getLoadedSimulationModelNames(int simID) {
		return simulationRuns.get(simID).getLoadedSimulationModelNames();
	}

	@Override
	public Map<String, Species> getLoadedSpecies(int simID) {
		return simulationRuns.get(simID).getLoadedSpecies();
	}

	@Override
	public Species[] getLoadedSpeciesArray(int simID) {
		return simulationRuns.get(simID).getLoadedSpeciesArray();
	}

	@Override
	public Parameter getLoadedParameter(int simID, String modelName,
			String parameter) {
		return simulationRuns.get(simID).getLoadedParameter(modelName,
				parameter);
	}

	@Override
	public Record getLastAvailableRecord(int simID) {
		if (simulationRuns.size() > simID)
			return simulationRuns.get(simID).getLastAvailableRecord();
		return null;
	}

	@Override
	public Record getRecord(int simID) {
		return simulationRuns.get(simID).getRecord();
	}

	@Override
	public SimulationRun getSimulationRun(int simID) {
		return simulationRuns.get(simID);
	}

	@Override
	public void setSimulationRun(int simID, SimulationRun sr) {
		simulationRuns.put(simID, sr);
		// TODO
		firePropertyChange(simID, VRootController.SIMULATION_RUN_CREATED, null, false);
	}

	public void SSCConnected(int simID, Boolean onOff) {
		firePropertyChange(simID, VRootController.SSC_CONNECTED, null, onOff);
	}

	@Override
	public double getTimestep(int simID) {
		return simulationRuns.get(simID).getTimestep();
	}

	@Override
	public int getSimulationRunCount() {
		return simulationRuns.size();
	}

	@Override
	public int getRecordCount(final int simID) {
		try {
			return simulationRuns.get(simID).getRecordStore().size();
		} catch (Exception e) {
			// TODO Auto-generated catch block
			// e.printStackTrace();
			return -1;
		}
	}

	@Override
	public boolean hasSimID(int simID) {
		return simulationRuns.containsKey(simID);
	}

	@Override
	public void newSimulationRun(int simID, RecordStoreActionListener rsal) {
		SimulationRun nsr = new SimulationRun(rsal, simID);
		if (simulationRuns.containsKey(simID)) {
			nsr.setAvalableSimulationModels(simulationRuns.get(simID).getAvailableSimulationModels());
			simulationRuns.remove(simID);
		}
		simulationRuns.put(simID, nsr);
	}
}
