package rootgui.model.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.Set;

import rootgui.model.Parameter;


/**
 * model of a simulation
 */
public class SimulationModel implements Cloneable, Serializable {
	/**
	 * 
	 */
	private static final long serialVersionUID = -4253511832351633206L;
	
	/**
	 * name of the model
	 */
	String name;
	
	/**
	 * related species
	 */
	List<Species> species;
	
	/**
	 * parameters (referenced by name) of the model
	 */
	Map<String, Parameter> parameters;

	public SimulationModel(String modelName) {
		this(modelName, null, null);
	}

	public SimulationModel(String modelName, Species[] species) {
		this(modelName, species, null);
	}

	public SimulationModel(String modelName, Species[] species, Map<String, Parameter> parameters) {
		this.name = modelName;
		this.species = new ArrayList<Species>();
		if (species != null)
			for (Species s : species)
				this.species.add(s);

		this.parameters = new HashMap<String, Parameter>();
		if (parameters != null)
			setParameters(parameters);
	}

	public String getName() {
		return name;
	}

	/*
	 * Returns a Map<String, Parameter> of stored parameters
	 */
	public Map<String, Parameter> getParameters() {
		return parameters;
	}

	/*
	 * Returns a parameter value
	 */
	public String getParameterValue(String parameterName) {
		if (parameters.containsKey(parameterName))
			return parameters.get(parameterName).getValue();
		return null;
	}

	/*
	 * 
	 */
	public void setParameters(Map<String, Parameter> params) {
		parameters.clear();
		for (String pName : params.keySet()) {
			parameters.put(pName, params.get(pName));//new Parameter(pName, params.get(pName)));
		}
	}

	// public void setSpeciesNames(Species.Type speciesType, String[]
	// speciesNames) {
	// if (this.species == null) {
	// this.species = new HashMap<Species.Type, String[]>();
	// }
	// this.species.put(speciesType, speciesNames);
	// }

	public void addSpecies(Species s) {
		this.species.add(s);
	}

	/*
	 * Returns all species loaded in the model, irrespective of their type
	 */
	public List<Species> getSpecies() {
		return species;
	}

//	public String[] getSpeciesNames(Species.Type speciesType) {
//		return species.get(speciesType);
//	}
//	//
	// public Set<Species.Type> getSpeciesTypes() {
	// return species.keySet();
	// }

	public void setParameter(Parameter par) {
		parameters.put(par.getName(), par);
	}

	public void setParameter(String parName, String parValue) {
		parameters.put(parName, new Parameter(parName, parValue));
	}

	@Override
	public SimulationModel clone() throws CloneNotSupportedException {
		SimulationModel clone = new SimulationModel(name);

		for (Species s : species)
			clone.addSpecies(s);
		Map<String, Parameter> pars =new HashMap<String, Parameter>();
		for (String m : parameters.keySet()) {
			pars.put(m, parameters.get(m));
		}
		clone.setParameters(pars);
		return clone;
	}

	@Override
	public String toString() {
		return name;
	}
}
