package rootgui.model.objects;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.Arrays;
import java.util.LinkedList;
import java.util.List;

/**
 * to describe a cell wall
 */
public class Wall implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = -5342580436224520857L;
	
	/**
	 * list of points that this wall is built with
	 */
	private List<Point> points;
	
	/**
	 * if this wall is an edge wall (i.e. outerest cell wall)
	 */
	private boolean edgeWall;
	
	/**
	 * id of the group that this wall belongs to 
	 */
	private int groupId;
	
	/**
	 * id of this wall
	 */
	private int id;
	
	/**
	 * id of cell that this wall belongs to
	 * Note, there could be more than one cellIDs in the future
	 */
	private int cellID;

	public Wall(List<Point> points) {
		this.points = new ArrayList<Point>(points);
		this.edgeWall = false;
		this.groupId = -1;
		id = -1;
		cellID = -1;
	}

	public Wall(Wall wall) {
		this.points = new ArrayList<Point>(wall.points);
		this.edgeWall = wall.edgeWall;
		this.groupId = wall.groupId;
		this.id = wall.getId();
		this.cellID = wall.getCellID();
	}

	public void setCellID(int id) {
		cellID = id;
	}

	public int getCellID() {
		return cellID;
	}

	public boolean getEdgeWall() {
		return edgeWall;
	}

	public void setEdgeWall(boolean edgeWall) {
		this.edgeWall = edgeWall;
	}

	public int getGroup() {
		return groupId;
	}

	public void setGroup(int group) {
		this.groupId = group;
	}

	public int getId() {
		return id;
	}

	public void setId(int id) {
		this.id = id;
	}

	public List<Point> getPoints() {
		return points;
	}

	
	/**
	 * @return a wall with points of current wall presented in a reverse order
	 */
	public Wall getReverse() {
		Wall newWall = new Wall(this);
		for (int i = 0; i < newWall.points.size() / 2; i++) {
			Point temp = newWall.points.get(i);
			newWall.points.set(i,
					newWall.points.get(newWall.points.size() - 1 - i));
			newWall.points.set(newWall.points.size() - 1 - i, temp);
		}
		return newWall;
	}

	public Point getHead() {
		return points.get(0);
	}

	public Point getTail() {
		if (points instanceof LinkedList) {
			return ((LinkedList<Point>) points).getLast();
		} else {
			return points.get(points.size() - 1);
		}
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof Wall) {
			Wall w = (Wall) obj;
			if (points.size() != w.points.size()) {
				return false;
			} else {
				for (int i = 0; i < points.size(); i++) {
					if (!points.get(i).equals(w.points.get(i))) {
						return false;
					}
				}
				return true;
			}
		} else {
			return false;
		}
	}

	@Override
	public int hashCode() {
		return points.size() * (getHead().hashCode() + getTail().hashCode());
	}

	@Override
	public String toString() {
		return points.toString();
	}

}
