package rootgui.model.parser;

import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.ArrayList;

import rootgui.model.parser.exceptions.UnexpectedCSVFileFormatException;

public class D2CSVparser {
	public static final String columnSeparator = ",";

	int[] cellIds;
	ArrayList<double[]> concentrationList;

	public D2CSVparser(String csvFile) {

		BufferedReader br = null;
		try {
			br = new BufferedReader(new FileReader(csvFile));

			String idsLine = br.readLine();
			String[] idSs = idsLine.split(columnSeparator);
			cellIds = new int[idSs.length];
			// concentrations = new double[valSs.length];

			for (int i = 0; i < idSs.length; i++) {
				cellIds[i] = Integer.parseInt(idSs[i]);
				// concentrations[i] = Double.parseDouble(valSs[i]);
			}

			concentrationList = new ArrayList<double[]>();
			String valueLine;
			while ((valueLine = br.readLine()) != null) {
				String[] valSs = valueLine.split(columnSeparator);
				if (idSs.length != valSs.length) {
					throw new UnexpectedCSVFileFormatException(
							"Columns mismatched");
				}

				double[] concentrations = new double[valSs.length];
				for (int i = 0; i < idSs.length; i++) {
					concentrations[i] = Double.parseDouble(valSs[i]);
				}

				concentrationList.add(concentrations);
			}
		} catch (FileNotFoundException e) {
			e.printStackTrace();
		} catch (IOException e) {
			e.printStackTrace();
		} finally {
			try {
				br.close();
			} catch (IOException e) {
				e.printStackTrace();
			}
		}

	}

	public int getSizeOfConcentrationRows() {
		return concentrationList.size();
	}

	public double[] getDataRow(int row) {
		return concentrationList.get(row);
	}

	public ArrayList<double[]> getData() {
		return concentrationList;
	}

	public int[] getCellIDs() {
		return cellIds;
	}

	// /*
	// * Sets the D2 concentrations in the model cells
	// */
	// public void getData() {
	// for (int index = renderIndexFrom; index <= renderIndexTo; index++) {
	// for (int i = 0; i < cellIds.length; i++) {
	// Cell c = model.getCellById(cellIds[i], index);
	// if (c == null) {
	// throw new FailToUpdateModelException(
	// "access to a non-existing cell in the model");
	// }
	// c.setConcentration(concentrationList.get(index)[i]);
	// }
	// }
	// }
}
