package rootgui.view;

import java.util.ArrayList;
import java.util.LinkedList;

import javax.swing.table.AbstractTableModel;

import rootgui.model.Parameter;
import rootgui.model.objects.SimulationModel;
import rootgui.view.settingspanel.DefaultSettingsPanelActionListener;

public class ParameterTableModel extends AbstractTableModel {
	String[] columnNames = { "Parameter", "Value" };
	ArrayList<Parameter> data;
	String modelName = "";
	LinkedList<DefaultSettingsPanelActionListener> listeners;
	
	public ParameterTableModel(LinkedList<DefaultSettingsPanelActionListener> listeners) {
		data = new ArrayList<Parameter>();
		this.listeners = listeners;
	}

	public void setModelParameters(SimulationModel sm) {
		modelName = sm.getName();
		data.addAll(sm.getParameters().values());
		fireTableDataChanged();
	}

	@Override
	public int getRowCount() {
		return data.size();
	}
	
	@Override
	public String getColumnName(int column) {
		return columnNames[column];
	}

	@Override
	public int getColumnCount() {
		return columnNames.length;
	}

	@Override
	public String getValueAt(int rowIndex, int columnIndex) {
		Parameter p = data.get(rowIndex);
		if (columnIndex == 0)
			return p.getName();
		else
			return p.getValue();
	}

	private boolean isInitialised = false;
	
	public boolean isSimInitialised(){
		return isInitialised;
	}
	
	public void setSimInitialised(boolean init){
		isInitialised =init;
	}
	
	public boolean isCellEditable(int row, int col) {
		return col == 1 && isInitialised;
	}

	public void setValueAt(Object value, int row, int col) {
		Parameter p = null;
		if (row < data.size())
			p = data.get(row);
		else {
			p = new Parameter(null, null);
			data.add(p);
		}
		if (col == 0)
			p.setName(value.toString());
		else
			p.setValue(value.toString());		
		fireTableCellUpdated(row, col);
		for (DefaultSettingsPanelActionListener l: listeners)
			l.fireParameterChanged(modelName, p);
	}

	/*
	 * Removes all data from the model
	 */
	public void clear() {
		data.clear();
		fireTableDataChanged();
	}

}
