package rootgui.view.settingspanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.beans.PropertyChangeEvent;
import java.io.Serializable;
import java.util.HashMap;
import java.util.LinkedList;
import java.util.Map;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.DefaultListModel;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JLabel;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListCellRenderer;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.DefaultTableModel;

import rootgui.controller.VRootController;
import rootgui.model.objects.SimulationModel;
import rootgui.model.objects.Species;
import rootgui.view.ParameterTableModel;
import visualiser.Figure;

/**
 * 
 * 
 */
public class DefaultSettingsPanel extends AbstractSettingsPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = -2921964326848243305L;

	/*
	 * Reference to the outer frame, into which this DefaultSettingsPanel is
	 * embeded
	 */
	final JPanel outerFrame;

	/*
	 * selected simulation model; could be more than one in the future
	 */
	SimulationModel selectedSimulationModel;

	/*
	 * the selected figure object from this settings panel
	 */
	Figure selectedFigure;

	/*
	 * the selected tissue (geometry) name
	 */
	String selectedTissueName;

	/*
	 * 
	 */
	LinkedList<DefaultSettingsPanelActionListener> listeners;

	/*
	 * simulation models available for selection
	 */
	DefaultListModel<SimulationModel> dlmModels;
	JList<SimulationModel> jlModels;

	JTable tbParameters;
	/*
	 * model of the table to display parameters of the selected model
	 */
	ParameterTableModel ptm;

	/*
	 * model of the table to display related species of the selected model
	 */
	DefaultTableModel stm;

	// combobox to select the tissue (geometry)
	JComboBox<String> jcbTissue;

	JButton btnSet, btnInitSim;

	private Map<String, String> setModels = new HashMap<String, String>();
	
	JTable tbSpecies;

	public DefaultSettingsPanel(final JPanel outerFrame,
			LeahsSettingsPanelActionListener initialListener) {
		/* PARAMETERS PANEL */
		setLayout(new GridBagLayout());
		this.listeners = new LinkedList<DefaultSettingsPanelActionListener>();
		if (initialListener != null)
			listeners.add(initialListener);
		this.outerFrame = outerFrame;		
		
		// setPreferredSize(new Dimension(300, 400));
		GridBagConstraints c = new GridBagConstraints();

		// tissue selection area
		c.gridx = 0;
		c.gridy = 0;
		c.gridwidth = 4;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.FIRST_LINE_START;
		c.weightx = 1;
		c.weighty = 0;
		// c.insets = new Insets(20, 0, 0, 0);
		if (javax.swing.UIManager.getLookAndFeel().getName().equals("Nimbus")) 
			c.ipady = -30;
		// c.ipadx=40;
		// String[] availableTissues = { "R12data_OA_wc.zip" };
		jcbTissue = new JComboBox<String>();
		jcbTissue.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createTitledBorder("Tissue Selection"), BorderFactory
				.createEmptyBorder(5, 5, 5, 5)));
		jcbTissue.addItem("");
		jcbTissue.addActionListener(new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {

				if (jcbTissue.isEnabled() && jcbTissue.getSelectedItem() != null && jcbTissue.getSelectedItem().toString().length() > 4)
					for (DefaultSettingsPanelActionListener l : listeners) {
						l.fireGeometryChanged(jcbTissue.getSelectedItem().toString());
					}
				if (setModels.size() > 0)
					btnInitSim.setEnabled(true);
			}

		});
		add(jcbTissue, c);

		// model selection area
		c.gridx = 0;
		c.gridy = 1;
		c.gridwidth = 4;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		c.weighty = 1;
		c.weightx = 1;
		// c.ipady = 50;
		// c.ipadx = 50;
		JPanel jpModels = createModelSelectionPanel();
		// jpModels.setBackground(Color.BLUE);
		jpModels.setBorder(BorderFactory.createCompoundBorder(BorderFactory
				.createTitledBorder("Model Selection"), BorderFactory
				.createEmptyBorder(5, 5, 5, 5)));
		add(jpModels, c);

		// set button
		c.fill = GridBagConstraints.HORIZONTAL;
		c.weightx = 1;
		c.weighty = 0;
		c.gridx = 0;
		c.gridy = 2;
		c.gridwidth = 2;
		btnSet = new JButton("Add Model");
		btnSet.setEnabled(false);
		btnSet.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {
				setSelectedModel();
			}
		});
		add(btnSet, c);

		// initialise tissue button
		c.gridx = 2;
		c.gridy = 2;
		c.fill = GridBagConstraints.HORIZONTAL;
		// c.anchor = GridBagConstraints.LINE_START;
		c.weightx = 1;
		c.weighty = 0;
		c.gridwidth = 2;
		btnInitSim = new JButton("Initialise");
		btnInitSim.setEnabled(false);
		btnInitSim.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent e) {

				for (DefaultSettingsPanelActionListener l : listeners) {
					l.fireInitSimButtonPressed();
				}
				ptm.setSimInitialised(true);
			}
		});
		add(btnInitSim, c);
	}

	private void setSelectedModel(){
		if (jlModels.getSelectedIndex() >= 0 && !setModels.containsKey(jlModels.getSelectedValue().getName())) {
			selectedSimulationModel = jlModels.getSelectedValue();
			for (DefaultSettingsPanelActionListener l : listeners) {
				l.fireSetButtonPressed(selectedSimulationModel);
			}
			setModels.put(selectedSimulationModel.getName(), selectedSimulationModel.getName());
			if (jcbTissue.getSelectedIndex() > 0)
				btnInitSim.setEnabled(true);
		}
	}
	/*
	 * 
	 */
	public SimulationModel getSelectedSimulationModel() {
		return selectedSimulationModel;
	}

	/*
	 * 
	 */
	public String getSelectedTissueName() {
		return selectedTissueName;
	}

	/*
	 * create the model selection panel
	 */
	private JPanel createModelSelectionPanel() {
		JPanel jpModels = new JPanel(new GridBagLayout());
		// jpModels.setPreferredSize(new Dimension(300, 400));
		GridBagConstraints c = new GridBagConstraints();

		// selection model
		c.gridx = 0;
		c.gridy = 0;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.BOTH;
		c.weightx = 1.0;
		c.weighty = 1;
		c.gridwidth = 4;
		c.ipady = 60;
		// c.ipadx = 70;
		// c.insets = new Insets(5, 0, 0, 0);

		dlmModels = new DefaultListModel<SimulationModel>();
		jlModels = new JList<SimulationModel>(dlmModels);
		JScrollPane spJlModels = new JScrollPane(jlModels);
		spJlModels
				.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
		spJlModels.setPreferredSize(new Dimension(250, 300));
		jlModels.setSelectionMode(ListSelectionModel.SINGLE_INTERVAL_SELECTION);
		jlModels.addListSelectionListener(new ListSelectionListener() {
			@Override
			public void valueChanged(ListSelectionEvent e) {
				ptm.clear();
				btnSet.setEnabled(true);
				if (jlModels.getSelectedValue() != null) {
					SimulationModel sm = (SimulationModel) jlModels
							.getSelectedValue();
					// SimulationModel sm =
					// controller.getAvailableSimulationModel(modelName);
					ptm.setModelParameters(sm);
					stm.setRowCount(0);
					for (Species s : sm.getSpecies()) {
						Vector<Species> row = new Vector<Species>();
						row.add(s);
						stm.addRow(row);
					}
				}
			}
		});
		jlModels.addMouseListener(new MouseListener() {

			@Override
			public void mouseReleased(MouseEvent e) {
			}

			@Override
			public void mousePressed(MouseEvent e) {
			}

			@Override
			public void mouseExited(MouseEvent e) {
			}

			@Override
			public void mouseEntered(MouseEvent e) {
			}

			@Override
			public void mouseClicked(MouseEvent e) {
				if (e.getClickCount() == 2) {
					setSelectedModel();							
				}
			}
		});
		// spJlModels.setBorder(BorderFactory.createCompoundBorder(
		// BorderFactory.createTitledBorder("Model Seletions"),
		// BorderFactory.createEmptyBorder(5, 5, 5, 5)));
		jpModels.add(spJlModels, c);

		// parameter table
		c.gridx = 0;
		c.gridy = 1;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.LINE_START;
		c.ipadx = 50;
		c.gridwidth = 3;
		c.weightx = 1;
		c.weighty = 0;
		JPanel parameterTablePanel = new JPanel(new BorderLayout());
		parameterTablePanel.add(new JLabel("Parameters: "), BorderLayout.NORTH);
		ptm = new ParameterTableModel(listeners);
		tbParameters = new JTable(ptm);
		// tbParameters.setPreferredSize(new Dimension(200, 100));
		JScrollPane spTbParams = new JScrollPane(tbParameters);
		// spTbParams.setSize(new Dimension(200,100));
		parameterTablePanel.add(spTbParams, BorderLayout.CENTER);
		ptm.addTableModelListener(new TableModelListener() {
			@Override
			public void tableChanged(TableModelEvent e) {
				try {
					SimulationModel sm = dlmModels.getElementAt(e.getLastRow());
					sm.setParameter(ptm.getValueAt(e.getLastRow(), 0), ptm
							.getValueAt(e.getLastRow(), 1));

				} catch (Exception e1) {
					// e1.printStackTrace();
				}
			}
		});
		jpModels.add(parameterTablePanel, c);

		// species table panel
		c.gridx = 3;
		c.gridy = 1;
		c.ipadx = 0;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.anchor = GridBagConstraints.LINE_START;
		c.gridwidth = 1;
		c.weightx = 1;
		JPanel speciesTablePanel = new JPanel(new BorderLayout());
		speciesTablePanel.add(new JLabel("Species: "), BorderLayout.NORTH);
		stm = new DefaultTableModel(0, 1);
		tbSpecies = new JTable(stm);
		// speciesTable.setSize(new Dimension(200, 100));
		JScrollPane spSpeciesTable = new JScrollPane(tbSpecies);
		// speciesTable.setPreferredSize(new Dimension(200,200));
		speciesTablePanel.add(spSpeciesTable, BorderLayout.CENTER);
		jpModels.add(speciesTablePanel, c);

		return jpModels;
	}

	// /**
	// * Create the GUI and show it. For thread safety, this method should be
	// * invoked from the event dispatch thread.
	// */
	// private static void createAndShowGUI() {
	// // Create and set up the window.
	// JFrame frame = new JFrame("DefaultSettingPanelDemo");
	// frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	//
	// // Add content to the window.
	// frame.add(new DefaultSettingsPanel(frame, null), BorderLayout.CENTER);
	//
	// // Display the window.
	// frame.pack();
	// frame.setVisible(true);
	// }
	//
	// public static void main(String[] args) {
	// // Schedule a job for the event dispatch thread:
	// // creating and showing this application's GUI.
	// SwingUtilities.invokeLater(new Runnable() {
	// public void run() {
	// // Turn off metal's use of bold fonts
	// UIManager.put("swing.boldMetal", Boolean.FALSE);
	// createAndShowGUI();
	// }
	// });
	// }

	@Override
	public void addSettingPanelActionListener(
			DefaultSettingsPanelActionListener l) {
		listeners.add(l);
	}

	@Override
	public void removeSettingPanelActionListener(
			DefaultSettingsPanelActionListener l) {
		listeners.remove(l);
	}

	public Figure getFigure() {
		return selectedFigure;
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(
				VRootController.AVAILABLE_SIMULATION_MODEL)) {
			SimulationModel sm = (SimulationModel) evt.getNewValue();
			dlmModels.add(dlmModels.size(), sm);
		} else if (evt.getPropertyName().equals(
				VRootController.AVAILABLE_TISSUE_FILE)) {
			String tissueFile = evt.getNewValue().toString();
			jcbTissue.addItem(tissueFile);
		}
	}

	@Override
	public void setEnabled(boolean onOff) {
		jlModels.setEnabled(onOff);
		jcbTissue.setEnabled(onOff);
		tbParameters.setEnabled(onOff);
	}

	@Override
	public Map<String, Serializable> getConfigurationMap() {
		map.put("jlModels.isEnabled()", jlModels.isEnabled());		
		map.put("jlModels.getSelectedIndex()", jlModels.getSelectedIndex());
		
		map.put("tbParameters.isEnabled()", tbParameters.isEnabled());
		
		map.put("jcbTissue.isEnabled()", jcbTissue.isEnabled());
		map.put("jcbTissue.isEditable()", jcbTissue.isEditable());
		map.put("jcbTissue.getSelectedIndex()", jcbTissue.getSelectedIndex());
		
		map.put("btnSet.isEnabled()", btnSet.isEnabled());
		map.put("btnInitSim.isEnabled()", btnInitSim.isEnabled());		
		
		map.put("setModels", (Serializable)setModels);
		
		map.put("ptm.isSimInitialised()", ptm.isSimInitialised());
		return map;
	}

	@Override
	public void setConfigurationMap(Map<String, Serializable> map) {
		super.setConfigurationMap(map);
		try {
			jlModels.setEnabled((Boolean) map.get("jlModels.isEnabled()"));
			jlModels.setSelectedIndex((Integer) map.get("jlModels.getSelectedIndex()"));
						
			jcbTissue.setEnabled(false);
			jcbTissue.setEditable((Boolean) map.get("jcbTissue.isEditable()"));
			jcbTissue.setSelectedIndex((Integer) map.get("jcbTissue.getSelectedIndex()"));
			jcbTissue.setEnabled((Boolean) map.get("jcbTissue.isEnabled()"));
									
			tbParameters.setEnabled((Boolean) map.get("tbParameters.isEnabled()"));
			
			btnSet.setEnabled((Boolean) map.get("btnSet.isEnabled()"));
			btnInitSim.setEnabled((Boolean) map.get("btnInitSim.isEnabled()"));
			
			ptm.setSimInitialised((Boolean) map.get("ptm.isSimInitialised()"));
			setModels = (Map<String, String>)map.get("setModels");
		} catch (Exception e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	@Override
	public void triggerNewSimulation() {
		if (jcbTissue.getSelectedItem() != null && ((String)jcbTissue.getSelectedItem()).length()>4)
		for (DefaultSettingsPanelActionListener l : listeners) {
			l.fireGeometryChanged(jcbTissue.getSelectedItem().toString());
		}
		btnInitSim.setEnabled(false);
		ptm.setSimInitialised(false);
		setModels.clear();
	}

	@Override
	public void reset() {
		super.reset();
		jcbTissue.removeAllItems();
		jcbTissue.addItem("");
		ptm.clear();
		dlmModels.clear();
		setModels.clear();
		stm.setRowCount(0);
		ptm.setSimInitialised(false);
		btnInitSim.setEnabled(false);
	}
}
