package rootgui.view.settingspanel;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.GridLayout;
import java.awt.SystemColor;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.KeyEvent;
import java.beans.PropertyChangeEvent;
import java.io.Serializable;
import java.util.LinkedList;
import java.util.Map;

import javax.swing.ButtonGroup;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JCheckBox;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextField;
import javax.swing.ScrollPaneConstants;
import javax.swing.SwingConstants;
import javax.swing.ToolTipManager;

import rootgui.controller.VRootController;
import rootgui.model.objects.SimulationModel;
import visualiser.Figure;

public class LeahsSettingsPanel extends AbstractSettingsPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 2489350895142754323L;

	/**
	 * name of model used by this settings panel
	 */
	public static final String MODEL_AT_CWD = "at_cwd";

	/**
	 * name of model used by this settings panel
	 */
	public static final String MODEL_DIIQ = "DIIq";

	/**
	 * reference to the tabbed settings panels
	 */
	private JPanel pnParameters, pnGeometry, pnCarriers; // pnPresets,

	/**
	 * reference to presets buttons
	 */
	// private JRadioButton[] rbsPresets = { new JRadioButton("Wild Type"),
	// new JRadioButton("aux1 Knockout"), new JRadioButton("NPA Treated") };

	/**
	 * carrier parameters
	 */
	private final String[] carrierParams = { "S1S2S3_AUX1", "LRC1LRC2_AUX1",
			"LRC3LRC4LRC5_AUX1", "elongepi_AUX1", "meriepi_AUX1",
			"elongcor_AUX1", "stele_LAX", "QCinit_LAX", "S2_LAX",
			"periepi_PIN", "periLRC_PIN", "pericor_PIN" };
	// "QCinit_LAX"
	/**
	 * carrier check boxes
	 */
	private CarrierCheckBox[] carrierCheckBoxes = {
			new CarrierCheckBox("AUX1 in columella S1, S2 and S3 tiers",
					carrierParams[0]),
			new CarrierCheckBox("AUX1 in outer LRC", carrierParams[1]),
			new CarrierCheckBox("AUX1 in inner LRC", carrierParams[2]),
			new CarrierCheckBox("AUX1 in elongating epidermal cells",
					carrierParams[3]),
			new CarrierCheckBox("AUX1 in meristematic epidermal cells",
					carrierParams[4]),
			new CarrierCheckBox("AUX1 in elongating cortical cells",
					carrierParams[5]),
			new CarrierCheckBox("LAX in stele", carrierParams[6]),
			new CarrierCheckBox("LAX in QC and columella initials",
					carrierParams[7]),
			new CarrierCheckBox("LAX in S2 tier of columella", carrierParams[8]),
			new CarrierCheckBox("PIN on inner faces of epidermal cells",
					carrierParams[9]),
			new CarrierCheckBox("PIN on inner faces of LRC cells",
					carrierParams[10]),
			new CarrierCheckBox("PIN on inner faces of cortical cells",
					carrierParams[11]), };

	private boolean[] cbSelected = { true, true, true, true, true, true, true,
			true, true, false, false, false };

	/**
	 * parameter fields
	 */
	private ParameterField[] parameterFields = {
	/*
	 * permeability of membrane to protonated auxin, PIAAH
	 */
	new ParameterField("PIAAH", null, 0.0, 10.0),

	/*
	 * permeability of membrane to anionic auxin due to AUX1, PAUX1
	 */
	new ParameterField("PAUX", null, 0.0, 10.0),

	/* 
	 *  permeability due to LAX
	 */
	new ParameterField("PLAX", null, 0.0, 10.0),
	
	/*
	 * permeability of membrane to anionic auxin due to PINs, PPIN
	 */
	new ParameterField("PPIN", null, 0.0, 10.0),

	/* 
	 *  permeability due to PGP
	 */
	new ParameterField("PNPE", null, 0.0, 10.0),
	
	/*
	 * apoplastic pH
	 */
	new ParameterField("pHapo", null, 1.0, 10.0),

	/*
	 * cytoplasmic pH
	 */
	new ParameterField("pHcyt", null, 1.0, 10.0),

	/*
	 * membrane potential
	 */
	new ParameterField("Voltage", null, -100.0, 100.0),

	/*
	 * auxin production rate
	 */
	new ParameterField("alpha_back", null, 0.0, 100.0),

	/*
	 * auxin production rate in columella initials and QC cells t
	 */

	new ParameterField("alpha_QCinit", null, 0.0, 100.0),

	/*
	 * auxin degradation rate
	 */
	new ParameterField("beta", null, 0.0, 100.0),

	/*
	 * cell-wall thickness
	 */
	new ParameterField("lamb", null, 0.001, 10.0) };

	/**
	 * listeners
	 */
	private LinkedList<LeahsSettingsPanelActionListener> listeners;

	/**
	 * the reset to default button in the parameter settings panel
	 */
	private JButton resetToDefault;

	JRadioButton jrbWTg1 = new JRadioButton("Geometry 1");
	JRadioButton jrbWTg2 = new JRadioButton("Geometry 2");
	JRadioButton jrbWTg3 = new JRadioButton("Geometry 3");
	JRadioButton jrbAUXg1 = new JRadioButton("Geometry 1");
	JRadioButton jrbAUXg2 = new JRadioButton("Geometry 2");
	JRadioButton jrbAUXg3 = new JRadioButton("Geometry 3");
	JRadioButton jrbJ0951g1 = new JRadioButton("Geometry 1");
	JRadioButton jrbJ0951g2 = new JRadioButton("Geometry 2");
	JRadioButton jrbJ0951g3 = new JRadioButton("Geometry 3");
	/**
	 * radio buttons for geometry selections
	 */
	private JRadioButton[] geoButtons = { jrbWTg1, jrbWTg2, jrbWTg3, jrbAUXg1,
			jrbAUXg2, jrbAUXg3, jrbJ0951g1, jrbJ0951g2, jrbJ0951g3 };

	private JTabbedPane tabbedPane;

	/*
	 * Artifact used in recording the two simulation models that Leah's
	 * simulation uses. These are captured through the modelPropertyChange
	 * function
	 */
	private SimulationModel[] toloadSM = new SimulationModel[2];
	/**
	 * geometery file names
	 */
	static final String[] geoFileNames = {

		"R12data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip",
		"R1data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip",
		"R3data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip",
		"R3aux1data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip",
		"R4aux1data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip",
		"R12aux1data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip",
		"R1_J951data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip",
		"R2_J951data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip",
		"R4_J951data_wsn_6pix_OA_fl_wceinit_wAUX1PINLAX_wc_wxauto_wdata_scaled_newvasc_adj.zip"
	};
		
	// number of wildtype geometries
	private int nWT = 3;

	// number of aux1 geometries
	private int nAUX = 3;

	// number of aux1 x J0951 >> AUX1 geometries
	private int nJ0951 = 3;

	/*
	 * used to categorised data of stored configurations
	 */
	public final static String GEOMETRY = "LeahsSettingsPanel.geometry";
	public final static String GEO_ENABLED = "LeahsSettingsPanel.geo_enabled";
	public final static String CARRIERS = "LeahsSettingsPanel.carriers";
	public final static String CARRIERS_ENABLED = "LeahsSettingsPanel.carriers_enabled";
	public final static String PARAMETERS = "LeahsSettingsPanel.parameters";
	
//	public final static String MODELS_LOADED = "LeahsSettingsPanel.models_loaded";


	/*
	 * ButtonGroup for the Presets buttons
	 */
	// private ButtonGroup preSetsGroup;
	/*
	 * ButtonGroup for the geometry radio buttons
	 */
	private ButtonGroup geoGroup;

	private boolean defaults_set = false;
	private boolean models_loaded = false;
	

	public LeahsSettingsPanel(JPanel outerFrame,
			LeahsSettingsPanelActionListener initialListener) {
		listeners = new LinkedList<LeahsSettingsPanelActionListener>();
		setLayout(new BorderLayout());

		setPreferredSize(new Dimension(250, 400));
		initComponents();
		if (initialListener != null)
			listeners.add(initialListener);
		ToolTipManager.sharedInstance().setInitialDelay(0);
	}

	private void initComponents() {
		tabbedPane = new JTabbedPane();
		tabbedPane.setTabLayoutPolicy(JTabbedPane.WRAP_TAB_LAYOUT);
		ImageIcon icon = null;// createImageIcon("images/middle.gif");

		pnGeometry = createGeometryPanel();
		tabbedPane.addTab("Geometry", icon, pnGeometry,
				"Select a tissue geometry");
		tabbedPane.setMnemonicAt(tabbedPane.getTabCount() - 1, KeyEvent.VK_3);

		pnParameters = createParameterPanel();
		tabbedPane.addTab("Parameters", icon, pnParameters,
				"Set model parameters");
		tabbedPane.setMnemonicAt(tabbedPane.getTabCount() - 1, KeyEvent.VK_2);

		pnCarriers = createCarrierPanel();
		tabbedPane.addTab("Carriers", icon, pnCarriers,
				"Specify carrier setings");
		tabbedPane.setMnemonicAt(tabbedPane.getTabCount() - 1, KeyEvent.VK_4);

		resetState();

		// Add the tabbed pane to this panel.
		add(tabbedPane, BorderLayout.CENTER);
	}

	private JPanel createParameterPanel() {
		JPanel mainParaPanel = new JPanel(new BorderLayout());

		JPanel paraPanel = new JPanel(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		c.gridy = 0;

		/*
		 * add parameter fields to panel
		 */
		addLabeledTextField("Permeability to IAAH (\u00b5m/s)",
				"Permeability of membrane to protonated auxin, PIAAH",
				parameterFields[0], paraPanel, c);
		addLabeledTextField("Permeability due to AUX1 (\u00b5m/s)",
				"Permeability of membrane to anionic auxin due to AUX1, PAUX1",
				parameterFields[1], paraPanel, c);
		addLabeledTextField("Permeability due to LAX (\u00b5m/s)",
				"Permeability of membrane to anionic auxin due to AUX1, PAUX1",
				parameterFields[2], paraPanel, c);
		addLabeledTextField("Permeability due to PINs (\u00b5m/s)",
				"Permeability of membrane to anionic auxin due to PINs, PPIN",
				parameterFields[3], paraPanel, c);
		addLabeledTextField("Permeability due to NPEs (\u00b5m/s)",
				"Permeability of membrane to anionic auxin due to non-polar efflux carriers, PNPE",
				parameterFields[4], paraPanel, c);
		addLabeledTextField("Apoplastic pH", null, parameterFields[5],
				paraPanel, c);
		addLabeledTextField("Cytoplasmic pH", null, parameterFields[6],
				paraPanel, c);
		addLabeledTextField("Membrane potential (V)", null, parameterFields[7],
				paraPanel, c);
		addLabeledTextField("Auxin production rate (/s)", null,
				parameterFields[8], paraPanel, c);
		addLabeledTextField("... in QC and col initials (/s)", null,
				parameterFields[9], paraPanel, c);
		addLabeledTextField("Auxin degradation rate (/s)", null,
				parameterFields[10], paraPanel, c);
		addLabeledTextField("Cell-wall thickness (\u00b5m)", null,
				parameterFields[11], paraPanel, c);

		JScrollPane scrollPane = new JScrollPane(paraPanel,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		// scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		mainParaPanel.add(scrollPane, BorderLayout.CENTER);

		/*
		 * listeners will be notified as soon as the parameter field loses focus
		 */
		FocusListener focusLis = new FocusListener() {
			double oldValue = -1;

			@Override
			public void focusLost(FocusEvent e) {
				ParameterField pf = (ParameterField) e.getSource();
				try {
					if (oldValue != Double.parseDouble(pf.getText()))
					{
						for (LeahsSettingsPanelActionListener l : listeners) {
							l.fireParameterChanged(MODEL_AT_CWD, pf.getParameter());
						}
					}
				} catch (NumberFormatException e1) {
					// TODO Auto-generated catch block
					// e1.printStackTrace();
				}
			}

			@Override
			public void focusGained(FocusEvent e) {
				ParameterField pf = (ParameterField) e.getSource();
				try {
					oldValue = Double.parseDouble(pf.getText());
				} catch (NumberFormatException e1) {
					// TODO Auto-generated catch block
					// e1.printStackTrace();
				}
			}
		};
		ActionListener actLis = new ActionListener() {
			@Override
			public void actionPerformed(ActionEvent e) {
				ParameterField pf = (ParameterField) e.getSource();
				// TODO Auto-generated method stub
				for (LeahsSettingsPanelActionListener l : listeners) {
					l.fireParameterChanged(MODEL_AT_CWD, pf.getParameter());
				}
			}
		};
		
		for (ParameterField pf : parameterFields) {
			pf.addFocusListener(focusLis);
			pf.addActionListener(actLis);
		}

		// panel to accommodate all buttons
		JPanel buttonPanel = new JPanel(new GridLayout(1, 1));

		/*
		 * the 'reset to default' button
		 */
		resetToDefault = new JButton();
		resetToDefault.setText("Reset to default");
		resetToDefault.addActionListener(new ActionListener() {

			@Override
			public void actionPerformed(ActionEvent arg0) {
				setParameterValuesToDefault();
			}
		});
		buttonPanel.add(resetToDefault);

		mainParaPanel.add(buttonPanel, BorderLayout.SOUTH);

		return mainParaPanel;
	}

	private JPanel createGeometryPanel() {
		JPanel geoRadioButtonPanel = new JPanel(new GridLayout(0, 1));

		/*
		 * add action listeners to each of the geometry buttion
		 */
		geoGroup = new ButtonGroup();
		geoRadioButtonPanel.add(new JLabel("  Wildtype"));

		for (int i = 0; i < nWT; i++) { // geoButtons.length
			geoGroup.add(geoButtons[i]);
			geoRadioButtonPanel.add(geoButtons[i]);
		}
		JPanel jpAUXlabel = new JPanel(new BorderLayout());
		JLabel jlAUX1 = new JLabel("  aux1");
		java.awt.Font f = jlAUX1.getFont();
		jlAUX1.setFont(new Font(f.getName(), Font.ITALIC, f.getSize() + 1));

		jpAUXlabel.add(jlAUX1, BorderLayout.WEST);
		// jpAUXlabel.add(new JLabel(" knockout"));
		geoRadioButtonPanel.add(jpAUXlabel, BorderLayout.CENTER);

		for (int i = nWT; i < nWT + nAUX; i++) {
			geoGroup.add(geoButtons[i]);
			geoRadioButtonPanel.add(geoButtons[i]);
		}

		JPanel jpJ0951label = new JPanel(new BorderLayout());
		JLabel jlJ0951 = new JLabel("  aux1 J0951 >> AUX1");
		jlJ0951.setFont(new Font(f.getName(), Font.ITALIC, f.getSize() + 1));

		jpJ0951label.add(jlJ0951, BorderLayout.WEST);
		// jpJ0951label.add(new JLabel(" knockout"));
		geoRadioButtonPanel.add(jpJ0951label, BorderLayout.CENTER);

		for (int i = nWT + nAUX; i < nWT + nAUX + nJ0951; i++) {
			geoGroup.add(geoButtons[i]);
			geoRadioButtonPanel.add(geoButtons[i]);
		}

		// adding event listeners to each geometry radio button
		for (int i = 0; i < geoButtons.length; i++) {
			final int index = i;
			geoButtons[index].addActionListener(new ActionListener() {
				int previousIndex = -1;

				@Override
				public void actionPerformed(ActionEvent e) {
					// if clicked twice, do nothing
					if (index == previousIndex)
						return;

					// generating geometry changed event
					for (LeahsSettingsPanelActionListener l : listeners) {
						l.fireGeometryChanged(geoFileNames[index]);
					}

					// setting carrier buttons
					if (index < nWT) {
						cbSelected[0] = true;
						cbSelected[1] = true;
						cbSelected[2] = true;
						cbSelected[3] = true;
						cbSelected[4] = false;
						cbSelected[5] = true;
						cbSelected[6] = true;
						cbSelected[7] = true;
						cbSelected[8] = true;
					} else if (nWT <= index && index < nWT + nAUX) {
						cbSelected[0] = false;
						cbSelected[1] = false;
						cbSelected[2] = false;
						cbSelected[3] = false;
						cbSelected[4] = false;
						cbSelected[5] = false;
						cbSelected[6] = true;
						cbSelected[7] = true;
						cbSelected[8] = true;
					} else if (nWT + nAUX <= index) {
						cbSelected[0] = false;
						cbSelected[1] = true;
						cbSelected[2] = false;
						cbSelected[3] = true;
						cbSelected[4] = false;
						cbSelected[5] = false;
						cbSelected[6] = true;
						cbSelected[7] = true;
						cbSelected[8] = true;
					}
					cbSelected[9] = false;
					cbSelected[10] = false;
					cbSelected[11] = false;

					// apply carrier selection
					for (int i = 0; i < carrierCheckBoxes.length; i++) {
						JCheckBox box = carrierCheckBoxes[i];
						box.setSelected(cbSelected[i]);
					}

					enableParamCarriers(true);

					// fire carrier selection events
					for (LeahsSettingsPanelActionListener l : listeners) {
						l.fireCarriersChanged(carrierParams, cbSelected);
					}

//					if (!models_loaded) {
					// load models
					for (LeahsSettingsPanelActionListener l : listeners) {
						l.fireSetButtonPressed(toloadSM[0]);
						l.fireSetButtonPressed(toloadSM[1]);
					}
//					models_loaded = true;
//					}
					// initialise parameter fields
					for (ParameterField pf : parameterFields)
						pf.setEnabled(true);
					initDefaultParameterValues(toloadSM[1]);

					// fire simulation initialised event
					for (LeahsSettingsPanelActionListener l : listeners) {
						l.fireInitSimButtonPressed();
					}
				}
			});
		}

		return geoRadioButtonPanel;
	}

	private JPanel createCarrierPanel() {
		JPanel carrierPanel = new JPanel(new GridLayout(12, 1));

		for (int i = 0; i < carrierCheckBoxes.length; i++) {
			JCheckBox box = carrierCheckBoxes[i];

			if (i == 0)
				box.setPreferredSize(new Dimension(30, 10));

			box.setSelected(cbSelected[i]);
			box.addActionListener(new ActionListener() {

				@Override
				public void actionPerformed(ActionEvent e) {
					if (e.getSource() instanceof CarrierCheckBox) {
						// boolean[] cbSelected = new
						// boolean[carrierParams.length];
						for (int i = 0; i < carrierCheckBoxes.length; i++)
							cbSelected[i] = carrierCheckBoxes[i].isSelected();
						for (LeahsSettingsPanelActionListener l : listeners) {
							l.fireCarriersChanged(carrierParams, cbSelected);
						}
					} else { // TODO PIN Checkbox; not sure how to deal with
								// this yet

						JCheckBox box = (JCheckBox) e.getSource();

					}
				}
			});
			carrierPanel.add(box);
		}
		JScrollPane scrollPane = new JScrollPane(carrierPanel,
				ScrollPaneConstants.VERTICAL_SCROLLBAR_AS_NEEDED,
				ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED);
		// scrollPane.setHorizontalScrollBarPolicy(ScrollPaneConstants.HORIZONTAL_SCROLLBAR_NEVER);
		JPanel mainCarrierPanel = new JPanel(new BorderLayout());

		mainCarrierPanel.add(scrollPane, BorderLayout.CENTER);

		return mainCarrierPanel;
	}

	/**
	 * @param label
	 *            , label before the field
	 * @param tooltip
	 *            , tool tip for the label
	 * @param field
	 *            , parameter field
	 * @param panel
	 *            , to which this field and label are added
	 * @param c
	 *            , the GridBagContraint used for the layout
	 */
	private void addLabeledTextField(String label, String tooltip,
			JTextField field, JPanel panel, GridBagConstraints c) {

		c.gridx = 0;
		c.gridy = c.gridy + 1;
		c.anchor = GridBagConstraints.LINE_START;
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridwidth = 1;
		JLabel jlabel = new JLabel(label, SwingConstants.LEFT);
		jlabel.setToolTipText(tooltip);
		// jlabel.setBorder(BorderFactory.createLineBorder(Color.BLACK));
		panel.add(jlabel, c);

		c.gridx = 1;
		c.anchor = GridBagConstraints.LINE_END;
		c.gridwidth = 1;
		c.ipadx = 0;
		// JTextField field = new JTextField();
		panel.add(field, c);
	}


	/*
	 * set all parameters in the parameter fields to default
	 */
	private void setParameterValuesToDefault() {
		for (ParameterField f : parameterFields) {
			f.setToDefaultValue();
			for (LeahsSettingsPanelActionListener l : listeners) {
				l.fireParameterChanged(MODEL_AT_CWD,f.getParameter());
			}	
		}

	}

	/** Returns an ImageIcon, or null if the path was invalid. */
	protected static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = LeahsSettingsPanel.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			System.err.println("Couldn't find file: " + path);
			return null;
		}
	}

	// /**
	// * Create the GUI and show it. For thread safety, this method should be
	// * invoked from the event dispatch thread.
	// */
	// private static void createAndShowGUI() {
	// // Create and set up the window.
	// JFrame frame = new JFrame("TabbedPaneDemo");
	// frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
	// JPanel jp = new JPanel();
	// frame.setLayout(new BorderLayout());
	// frame.add(jp, BorderLayout.CENTER);
	// // Add content to the window.
	// frame.add(new LeahsSettingsPanel(jp, null), BorderLayout.CENTER);
	//
	// // Display the window.
	// frame.pack();
	// frame.setVisible(true);
	// }

	// public static void main(String[] args) {
	// // Schedule a job for the event dispatch thread:
	// // creating and showing this application's GUI.
	// SwingUtilities.invokeLater(new Runnable() {
	// public void run() {
	// // Turn off metal's use of bold fonts
	// UIManager.put("swing.boldMetal", Boolean.FALSE);
	// createAndShowGUI();
	// }
	// });
	// }

	@Override
	public void addSettingPanelActionListener(
			DefaultSettingsPanelActionListener l) {
		listeners.add((LeahsSettingsPanelActionListener) l);
	}

	@Override
	public void removeSettingPanelActionListener(
			DefaultSettingsPanelActionListener l) {
		listeners.remove((LeahsSettingsPanelActionListener) l);
	}

	public Figure getFigure() {
		return null;
	}

	@Override
	public void modelPropertyChange(PropertyChangeEvent evt) {
		if (evt.getPropertyName().equals(
				VRootController.AVAILABLE_SIMULATION_MODEL)) {
			SimulationModel sm = (SimulationModel) evt.getNewValue();
			// System.err.println(sm.getName());
			if (sm.getName().equals(MODEL_DIIQ)) {
				toloadSM[0] = sm;
			} else if (sm.getName().equals(MODEL_AT_CWD)) {
				toloadSM[1] = sm;
			}
		} else if (evt.getPropertyName().equals(
				VRootController.SIMULATION_IN_PROGRESS)) {
			if ((Boolean) evt.getNewValue()) {
				setEnabled(false);
			}
		}
	}

	/*
	 * Initialise default parameter values for a given model
	 */
	private void initDefaultParameterValues(SimulationModel model) {
		if (model != null) {
			if (defaults_set == false) {

			String v;
			parameterFields[0].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("PIAAH")) != null ? v : "0"));
			parameterFields[1].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("PAUX")) != null ? v : "0"));
			parameterFields[2].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("PLAX")) != null ? v : "0"));
			parameterFields[3].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("PPIN")) != null ? v : "0"));
			parameterFields[4].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("PNPE")) != null ? v : "0"));
			parameterFields[5].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("pHapo")) != null ? v : "0")); // ?
			parameterFields[6].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("pHcyt")) != null ? v : "0")); // ?
			parameterFields[7].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("Voltage")) != null ? v : "0")); // ?
			parameterFields[8].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("alpha_back")) != null ? v : "0"));
			parameterFields[9].setDefaultValue(Double.parseDouble((v = model
							.getParameterValue("alpha_QCinit")) != null ? v : "0"));
			parameterFields[10].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("beta")) != null ? v : "0"));
			parameterFields[11].setDefaultValue(Double.parseDouble((v = model
					.getParameterValue("lamb")) != null ? v : "0"));
			defaults_set=true;
			for (ParameterField f : parameterFields) {
				f.setToDefaultValue();
			}
			} else
				setParameterValuesToDefault();
		}
	}

	@Override
	public void setEnabled(boolean onOff) {
		// TODO Auto-generated method stub
		// pnPresets.setEnabled(onOff);
		// for (JRadioButton jrb : rbsPresets)
		// jrb.setEnabled(onOff);

		if (!onOff) {
			pnParameters.setEnabled(onOff);
			for (ParameterField pf : parameterFields)
				pf.setEditable(onOff);
			resetToDefault.setEnabled(onOff);

			pnGeometry.setEnabled(onOff);
			for (JRadioButton jrb : geoButtons)
				jrb.setEnabled(onOff);

			pnCarriers.setEnabled(onOff);
			for (CarrierCheckBox ccb : carrierCheckBoxes)
				ccb.setEnabled(onOff);
		}
	}

	public void enableParamCarriers(boolean onOff) {
		pnParameters.setEnabled(onOff);
		for (ParameterField pf : parameterFields)
			pf.setEditable(onOff);
		resetToDefault.setEnabled(onOff);

		pnCarriers.setEnabled(onOff);
		for (CarrierCheckBox ccb : carrierCheckBoxes)
			ccb.setEnabled(onOff);
	}

	@Override
	public Map<String, Serializable> getConfigurationMap() {
		map = super.getConfigurationMap();

		boolean notConfigured = true;

		// presets
		// for (int i = 0; i < rbsPresets.length; i++) {
		// if (rbsPresets[i].isSelected()) {
		// map.put(PRESETS, i);
		// notConfigured = false;
		// break;
		// }
		// }

		// geometry
		notConfigured = true;
		for (int i = 0; i < geoButtons.length; i++) {
			if (geoButtons[i].isSelected()) {
				map.put(GEOMETRY, i);
				notConfigured = false;
				break;
			}
		}
		if (notConfigured) {
			map.put(GEOMETRY, -1);
		}
		
	//	map.put(MODELS_LOADED, models_loaded);

		Boolean[] geoEnabled = new Boolean[geoButtons.length];
		for (int i = 0; i < geoButtons.length; i++) {
			geoEnabled[i] = geoButtons[i].isEnabled();
		}
		map.put(GEO_ENABLED, geoEnabled);
		/*
		 * parameters params[i][0] is the current value stored in
		 * parameterFields[i] params[i][1] is the default value stored in
		 * parameterFields[i]
		 */

		Serializable[][] params = new Serializable[parameterFields.length][3];
		for (int i = 0; i < parameterFields.length; i++) {
			params[i][0] = parameterFields[i].getText();
			params[i][1] = parameterFields[i].getDefaultValue();
			params[i][2] = parameterFields[i].isEditable();
		}
		map.put(PARAMETERS, params);

		// carriers
		Boolean[] cbSelected = new Boolean[carrierParams.length];
		for (int i = 0; i < carrierCheckBoxes.length; i++) {
			cbSelected[i] = carrierCheckBoxes[i].isSelected();
		}
		map.put(CARRIERS, cbSelected);
		Boolean[] cbEnabled = new Boolean[carrierParams.length];
		for (int i = 0; i < carrierCheckBoxes.length; i++) {
			cbEnabled[i] = carrierCheckBoxes[i].isEnabled();
		}
		map.put(CARRIERS_ENABLED, cbEnabled);

		return map;
	}

	@Override
	public void setConfigurationMap(Map<String, Serializable> map) {
		super.setConfigurationMap(map);

		Object o;

	//	o = map.get(MODELS_LOADED);
	//	if(o!=null && o instanceof Boolean) {
	//		models_loaded = (Boolean) o;
	//	}
		
		// load GEOMETRY configuration
		o = map.get(GEOMETRY);
		if (o != null && o instanceof Integer) {
			int geoIndex = (Integer) o;
			if (geoIndex == -1) {
				geoGroup.clearSelection();
			} else {
				geoButtons[geoIndex].setSelected(true);
			}
		} else {
			System.err.println("Fail to extract GEOMETRY configuration!");
		}
		o = map.get(GEO_ENABLED);
		if (o != null && o instanceof Boolean[]) {
			Boolean[] geoEnabled = (Boolean[]) o;
			for (int i = 0; i < geoButtons.length; i++) {
				geoButtons[i].setEnabled(geoEnabled[i]);
			}
		}

		o = map.get(PARAMETERS);
		if (o != null && o instanceof Serializable[][]) {
			Serializable[][] params = (Serializable[][]) o;
			for (int i = 0; i < parameterFields.length; i++) {
				parameterFields[i].setText((String) params[i][0]);
				parameterFields[i].setDefaultValue((Double) params[i][1]);
				parameterFields[i].setEditable((Boolean) params[i][2]);
				parameterFields[i].updateBackground();
			}
			resetToDefault.setEnabled((Boolean) params[0][2]);
		} else {
			System.err.println("Fail to extract PARAMETERS configuration!");
		}

		// load CARRIERS configuration
		o = map.get(CARRIERS);
		if (o != null && o instanceof Boolean[]) {
			Boolean[] cbSelected = (Boolean[]) o;
			for (int i = 0; i < carrierCheckBoxes.length; i++) {
				carrierCheckBoxes[i].setSelected(cbSelected[i]);
			}
		} else {
			System.err.println("Fail to extract CARRIERS configuration!");
		}
		// load CARRIERS configuration
		o = map.get(CARRIERS_ENABLED);
		if (o != null && o instanceof Boolean[]) {
			Boolean[] cbEnabled = (Boolean[]) o;
			for (int i = 0; i < carrierCheckBoxes.length; i++) {
				carrierCheckBoxes[i].setEnabled(cbEnabled[i]);
			}
		} else {
			System.err.println("Fail to extract CARRIERS configuration!");
		}

		// the user is not allowed to modify the settings panel (e.g. editing
		// parameters) after the configuration is loaded
		// setEnabled(false);
	}

	@Override
	public void triggerNewSimulation() {

	}

	private void resetState() {
		geoGroup.clearSelection();
		for (ParameterField pf : parameterFields) {
			pf.setToDefaultValue();
			pf.updateBackground();
			pf.setEditable(false);
		}

		resetToDefault.setEnabled(false);
		pnParameters.setEnabled(false);

		for (CarrierCheckBox ccb : carrierCheckBoxes) {
			ccb.setSelected(false);
			ccb.setEnabled(false);
		}
		for (int i = 0; i < geoButtons.length; i++) {
			geoButtons[i].setEnabled(true);
			geoButtons[i].setSelected(false);
		}
		tabbedPane.setSelectedIndex(0);
	}

	@Override
	public void reset() {
		super.reset();
		// preSetsGroup.clearSelection();
		resetState();
	}
}
