package visualiser;

import java.awt.Color;
import java.io.Serializable;

import rootgui.model.objects.Species;

/*
 * Represents the type of information that is to be rendered within one Visualiser tab
 */
public class Figure implements Serializable {

	/**
	 * 
	 */
	private static final long serialVersionUID = 7089648597996380357L;

	// private String[] titles;
	private Species[] mSpecies;
	private double[] mMinRange;
	private double[] mMaxRange;
	private double[] mTxty = { 0, 0 };
	private double mZoomRatio = 1;
	private int mNoCdpColumns = 1;
	private int[] mSelectedSpeciesIndex;
	
	/*
	 * Boolean flag the specifies if this is the first time the CellDrawingPanel
	 * is rendering a tissue (in which case, it may calculate what zoom factor
	 * to use, so that the image fits the confines of the CellDrawingPanel
	 */
	protected boolean mFirstVisualisation = true;

	public Figure(Species species, double minRange, double maxRange) {
		mSpecies = new Species[1];
		mSpecies[0] = species;
		mMinRange = new double[1];
		mMinRange[0] = minRange;
		mMaxRange = new double[1];
		mMaxRange[0] = maxRange;
		mSelectedSpeciesIndex = new int[] { 0 };
	}

	public Figure(Species[] species) {
		this.mSpecies = species;
		mMinRange = new double[species.length];
		mMaxRange = new double[species.length];
		mSelectedSpeciesIndex = new int[1];
		mSelectedSpeciesIndex[0] = 0;
		for (int i = 0; i < species.length; i++) {
			mMinRange[i] = 0;
			mMaxRange[i] = 0;
		}
	}

	public Figure(Species[] species, double[] minRanges, double[] maxRanges,
			int[] selectedIndexes) {
		this.mSpecies = species;
		this.mMinRange = minRanges;
		this.mMaxRange = maxRanges;
		mSelectedSpeciesIndex = selectedIndexes;

		// if (species.length > 0) {
		// // int selectedNO = 0;
		// for (int i = 0; i < species.length; i++) {
		// // if (species[i].isSelected())
		// // selectedNO++;
		// }
		// // selectedSpeciesIndex = new int[selectedNO];
		// }

		// int j = 0;
		// if (selectedSpeciesIndex != null && selectedSpeciesIndex.length > 0)
		// {
		// for (int i = 0; i < species.length; i++) {
		// if (species[i].isSelected()) {
		// selectedSpeciesIndex[j] = i;
		// j++;
		// }
		// }
		// }
	}

	public int getSpeciesCount() {
		if (mSpecies != null)
			return mSpecies.length;
		return 0;
	}

	public int getSelectedSpeciesCount() {
		if (mSelectedSpeciesIndex == null)
			return 0;
		else
			return mSelectedSpeciesIndex.length;
	}

	public int[] getSelectedSpeciesIndex() {
		return mSelectedSpeciesIndex;
	}

	public void setSelectedSpecies(int[] indexes) {
		mSelectedSpeciesIndex = indexes;
	}

	/*
	 * Sets the preferred number of columns for the rendering of the species
	 */
	public void setNoColumns(int noColumns) {
		if (noColumns < 1) {
			System.err
					.println("Error: number of columns cannot be smaller than 1.\n[Figure.setNoColumns()]");
		} else
			mNoCdpColumns = noColumns;
	}

	/*
	 * Gets the preferred number of columns for the rendering of the species
	 */
	public int getNoColumns() {
		return mNoCdpColumns;
	}

	/*
	 * 
	 */
	public Species getSpecies(int index) {
		if (index < mSpecies.length)
			return mSpecies[index];
		return null;
	}

	public Species[] getSpecies() {
		return mSpecies;
	}

	public void setSpecies(Species[] species) {
		this.mSpecies = species;
		mMinRange = new double[species.length];
		mMaxRange = new double[species.length];
		mSelectedSpeciesIndex = new int[1];
		mSelectedSpeciesIndex[0] = 0;
		for (int i = 0; i < species.length; i++) {
			mMinRange[i] = 0;
			mMaxRange[i] = 0;
		}
	}

	public double getMinRange(int index) {
		if (mMinRange.length > index)
			return mMinRange[index];
		return 0;
	}

	public double getMaxRange(int index) {
		if (mMaxRange.length > index)
			return mMaxRange[index];
		return 0;
	}

	public double[] getMinRanges() {
		return mMinRange;
	}

	public double[] getMaxRanges() {
		return mMaxRange;
	}

	public double[] getTxTy() {
		return mTxty;
	}

	public void setTxTy(double tx, double ty) {
		mTxty[0] = tx;
		mTxty[1] = ty;
	}

	public double getZoomRatio() {
		return mZoomRatio;
	}

	public void setZoomRatio(double zr) {
		mZoomRatio = zr;
	}
	
	public boolean isFirstVisualisation() {
		return mFirstVisualisation;
	}

	public void setFirstVisualisation(boolean ft) {
		mFirstVisualisation = ft;
	}
}
