package visualiser.viewpanel.view;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Graphics;
import java.awt.LayoutManager;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import java.text.NumberFormat;
import java.util.Arrays;
import java.util.Hashtable;

import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JFormattedTextField;
import javax.swing.JLabel;
import javax.swing.JSlider;
import javax.swing.JSpinner;
import javax.swing.JToolBar;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;
import javax.swing.text.NumberFormatter;

import rootgui.model.objects.Species;

/**
 * 
 * @author Lei Yu
 * 
 */
public class ControlBar extends JToolBar implements IControlBar {

	JSlider iterationSlider;
	JSlider speciesSlider;
	// JButton startButton;
	JButton playButton;
	JButton zoomIn;
	JButton zoomOut;
	JSpinner spinner;
	// JButton exportButton;
	JLabel jlIteration;
	JLabel speciesLabel;
	JLabel iterationLabel;
	JComboBox jcbColourBar;
	JComboBox jcbExport;
	Species[] species;

	private ImageIcon startIcon = new ImageIcon(this.getClass().getResource(
			"start.png"));
	private ImageIcon playIcon = new ImageIcon(this.getClass().getResource(
			"player_play.png"));
	private ImageIcon pauseIcon = new ImageIcon(this.getClass().getResource(
			"pause.png"));

	private ImageIcon zoomInIcon = new ImageIcon(this.getClass().getResource(
			"zoom_out.png"));
	private ImageIcon zoomOutIcon = new ImageIcon(this.getClass().getResource(
			"zoom_in.png"));

	private ImageIcon exportIcon = new ImageIcon(this.getClass().getResource(
			"document_export.png"));

	private final String[] colorTypes = { "RGB", "Green" };
	private final String[] exportTypes = { "Export one", "Export all" };

	public ControlBar() {
		initComponents();
		setButtonsEnabled(false);
	}

	private void initComponents() {
		iterationSlider = new JSlider();
		iterationSlider.setSnapToTicks(true);
		iterationSlider.setMinorTickSpacing(1);
		iterationSlider.setMajorTickSpacing(10);
		iterationSlider.setPaintTicks(true);
		iterationSlider.setValue(0);
		iterationSlider.setMaximum(0);
		iterationSlider.addChangeListener(new ChangeListener() {
			@Override
			public void stateChanged(ChangeEvent e) {
				int val = iterationSlider.getValue();
				int max = iterationSlider.getMaximum();
				jlIteration.setText(String.format("Step %2d/%d", val, max));
			}
		});
		
		speciesSlider = new JSlider();
		Dimension preDemention = speciesSlider.getPreferredSize();
		speciesSlider.setPreferredSize(new Dimension(preDemention.width / 2,
				preDemention.height));
		speciesSlider.setSnapToTicks(true);
		speciesSlider.setMajorTickSpacing(1);
		speciesSlider.setPaintTicks(true);
		speciesSlider.setValue(0);
		speciesLabel = new JLabel("Species");

		// startButton = new JButton(startIcon);
		// startButton.setActionCommand("Start");

		playButton = new JButton(playIcon);
		playButton.setActionCommand("Play");
		playButton.setToolTipText("Play the animation");

		zoomIn = new JButton(zoomInIcon);
		zoomIn.setActionCommand("-");
		zoomIn.setToolTipText("Zoom Out");
		zoomOut = new JButton(zoomOutIcon);
		zoomOut.setActionCommand("+");
		zoomOut.setToolTipText("Zoom In");
		
		SpinnerModel zoomModel = new SpinnerNumberModel(1, // initial value
				0, // min
				100, // max
				VisualiserPanel.ZOOM_INCREMENT); // step
		spinner = new JSpinner(zoomModel);
		spinner.setEditor(new JSpinner.NumberEditor(spinner, "#%"));
		spinner.setPreferredSize(new Dimension(45, 30));
	
		jlIteration = new JLabel("Step 0/0");
		jcbColourBar = new JComboBox(colorTypes);
		jcbColourBar.setPreferredSize(new Dimension(60, 30));

		jcbExport = new JComboBox(exportTypes);
		jcbExport.setActionCommand("Export");

		// addSeparator(new Dimension(20, 30));
		// add(startButton);
		add(playButton);
		addSeparator(new Dimension(10, 30));
		add(zoomIn);
		add(zoomOut);
		add(spinner);
		addSeparator(new Dimension(10, 30));
		add(jlIteration);
		// addSeparator(new Dimension(10, 30));
		add(iterationSlider);
		addSeparator(new Dimension(20, 30));
		add(speciesLabel);
		// addSeparator(new Dimension(10, 30));
		add(speciesSlider);
		addSeparator(new Dimension(10, 30));
		add(new JLabel("Colour Bar"));
		add(jcbColourBar);
		addSeparator(new Dimension(10, 30));
		// add(exportButton);
		add(jcbExport);

		// setLayout((LayoutManager) new ModifiedFlowLayout(FlowLayout.LEFT));
		setLayout((LayoutManager) new BoxLayout(this, BoxLayout.X_AXIS));
		// setFloatable(false);
		setRollover(true);
		setMinimumSize(new Dimension(200, 30));
	}

	public JButton getZoomInButton() {
		return zoomIn;
	}

	public JButton getZoomOutButton() {
		return zoomOut;
	}

	public JSlider getSlider() {
		return iterationSlider;
	}

	public void addPlayActionListener(ActionListener al) {
		playButton.addActionListener(al);
	}


	public void addZoomActionListener(ActionListener al) {
		zoomIn.addActionListener(al);
		zoomOut.addActionListener(al);
	}

	public void addIterationSliderChangeListner(MouseMotionListener al) {
		iterationSlider.addMouseMotionListener(al);
	}

	public void addSpeciesSliderChangeListener(MouseMotionListener al) {
		speciesSlider.addMouseMotionListener(al);
	}

	public void addExportActionListener(ActionListener al) {
		// exportButton.addActionListener(al);
		jcbExport.addActionListener(al);
	}

	public void addColourBarSelectionActionListener(ActionListener al) {
		jcbColourBar.addActionListener(al);
	}

	public void addZoomFactorChangeListener(ChangeListener cl) {
		spinner.addChangeListener(cl);
	}

	public void setPlayButtonEnabled(boolean onOff){
		playButton.setEnabled(onOff && zoomIn.isEnabled());
	}
	
	@Override
	public void setButtonsEnabled(boolean enabled) {
		playButton.setEnabled(enabled);
		zoomIn.setEnabled(enabled);
		zoomOut.setEnabled(enabled);
		iterationSlider.setEnabled(enabled);
		spinner.setEnabled(enabled);
		speciesSlider.setEnabled(enabled);
		jcbColourBar.setEnabled(enabled);
		jcbExport.setEnabled(enabled);
	}

	@Override
	public void setMaxIterations(int maxIterations) {
		if (maxIterations != iterationSlider.getMaximum())
			iterationSlider.setMaximum(maxIterations);
	}

	@Override
	public void setIteration(int iteration) {
		if (iteration > iterationSlider.getMaximum()) {
			iterationSlider.setMaximum(iteration);
		}
		if (iteration >= iterationSlider.getMinimum() && iteration <= iterationSlider.getMaximum())
			if (iteration != iterationSlider.getValue())
				iterationSlider.setValue(iteration);
//			else 
//			iterationSlider.setValue(0);
	}

	@Override
	public void setSpecies(Species[] species) {
		setSpecies(species, 0);
	}

	@Override
	public void setSpecies(Species[] species, int selectedIndex) {
		//TODO replace this simple sort with a sort based on the species type first, then the species name
		if (species == null)
			species = new Species[]{};
		Arrays.sort(species);
		this.species = species;
		speciesSlider.setMinimum(0);
		speciesSlider.setMaximum(species.length - 1 > 0 ? species.length - 1 : 0);
		if (selectedIndex > species.length)
			selectedIndex = species.length - 1;
		else if (selectedIndex < 0)
			selectedIndex = 0;
		speciesSlider.setValue(selectedIndex);
	}

	public Species getSelectedSpecies(){
		try {
			return species[speciesSlider.getValue()];
		} catch (Exception e) {
			return null;
		}
	}
	
	public int getSelectedSpeciesIndex(){
		return speciesSlider.getValue();
	}
	
	public int getCurrentIteration(){
		return iterationSlider.getValue();
	}
	public int getMaxIterations() {
		return iterationSlider.getMaximum();
	}

	public void setZoomFactor(double zoomFactor) {
		spinner.setValue(zoomFactor);
	}
}
