package visualiser.viewpanel.view.cellrendering;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Stroke;
import java.awt.geom.AffineTransform;
import java.util.Collection;

import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import rootgui.model.recordstore.SpeciesRecord;

import visualiser.Figure;
import visualiser.viewpanel.view.CellDrawingPanel;
import visualiser.viewpanel.view.CellView;

/*
 * Abstract class specifying the operations that an object capable of rendering a cell, should perform
 */
public abstract class AbsCellRenderer {

	/*
	 * The Figure object from which the CellRenderer will take its information
	 * e.g. which species to render and what are the ranges
	 */
	protected Figure figure;
	/*
	 * The index within the Figure object, specifying what species it is
	 * actually rendering
	 */
	protected int figureIndex;

	/*
	 * The SpeciesRecord object from which the cell renderer will take its data
	 */
	protected SpeciesRecord sr;

	/*
	 * Reference to the parent CellDrawingPanel
	 */
	protected CellDrawingPanel cdp;

	public void setFigure(Figure figure, int figureIndex, Record r) {
		this.figure = figure;
		this.figureIndex = figureIndex;
		Species s = figure.getSpecies(figureIndex);
		try {
			if (r != null)
			sr = r.getData(figure.getSpecies(figureIndex));
			// if (sr == null)
			// System.err.println("Abcell render.newIteration: sr for species <"
			// + s + "> is null!");
		} catch (Exception e) {
//			 TODO Auto-generated catch block
			 e.printStackTrace();
		}
	}

	public void setCellDrawingPanel(CellDrawingPanel cdp) {
		this.cdp = cdp;
	}

	/*
	 * Signals that a new iteration is available. May lead to further
	 * calculations.
	 */
	public void newIteration(Record r) {
		Species s = figure.getSpecies(figureIndex);
		if (r != null)
			sr = r.getData(s);
			else 
			sr = null;
		// if (sr == null)
		// System.err.println("Abcell render.newIteration: sr for species <"
		// + s + "> is null!");
	};

	/*
	 * Draws a CellView object
	 */
	public abstract void drawCells(Collection<CellView> cvs, Graphics g);

	/*
	 * Highlights a CellView object
	 */
	public static void highlightCells(Collection<CellView> cvs, Graphics g) {
		if ( cvs == null)
			return;
		Graphics2D g2 = (Graphics2D) g;
		Stroke s = g2.getStroke();
		Color c = g2.getColor();
		g2.setStroke(CellView.dashed);
		g2.setColor(Color.green);

		for (CellView cv : cvs) {
			if (cv != null)
				g2.draw(cv.getPolygon());
		}

		g2.setStroke(s);
		g2.setColor(c);
	}
}
