package visualiser.viewpanel.view.cellrendering;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.RenderingHints;
import java.awt.Stroke;
import java.awt.geom.Path2D;
import java.util.Collection;

import visualiser.viewpanel.view.CellView;

/*
 * This cell renderer just draws the geometry (contour) of the tissue
 */
public class GeometryCellRenderer extends AbsCellRenderer {
	private Color contourColour;
	private Stroke stroke;

	public GeometryCellRenderer() {
		this(Color.BLACK, CellView.stroke);
	}

	public GeometryCellRenderer(Color contourColour, Stroke stroke) {
		this.contourColour = contourColour;
		if (this.contourColour == null)
			this.contourColour = Color.BLACK;
		this.stroke = stroke;
		if (this.stroke == null)
			this.stroke = CellView.stroke;
	}

	@Override
	public void drawCells(Collection<CellView> cvs, Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHint(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);

		// saving the state
		Color c = g2.getColor();
		Stroke s = g2.getStroke();

		for (CellView cv : cvs) {
			// drawing
			Path2D polygon = cv.getPolygon();
			g2.setStroke(stroke);
			g2.setColor(contourColour);
			g2.draw(polygon);
		}

		// restoring the state
		g2.setColor(c);
		g2.setStroke(s);
	}

}
