package visualiser.viewpanel.view.tabs;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridLayout;
import java.util.ArrayList;
import java.util.List;

import javax.swing.ImageIcon;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import rootgui.model.objects.Cell;
import rootgui.model.objects.Species;
import rootgui.model.recordstore.Record;
import visualiser.Figure;
import visualiser.IVisualiser;
import visualiser.viewpanel.controller.CDPController;
import visualiser.viewpanel.view.CellDrawingPanel;
import visualiser.viewpanel.view.VisualiserPanel;

//import com.sun.media.sound.FFT;

public class VisualiserTab extends JPanel {

	/**
	 * 
	 */
	private static final long serialVersionUID = 1L;
	public static final ImageIcon addIcon = new ImageIcon(
			VisualiserTab.class.getResource("add.gif"));
	public static final ImageIcon closeIcon = new ImageIcon(
			VisualiserTab.class.getResource("cross.gif"));

	/*
	 * The current Record being rendered
	 */
	private Record mRecord;

	/*
	 * A List refering to the VisualiserPanel objects contained within the
	 * VisualiserTab
	 */
	private List<VisualiserPanel> mVisPanels;

	/*
	 * A reverence to the IVisualiser object within which the VisualiserTab is
	 * contained
	 */
	private IVisualiser mvis;

	/*
	 * The Figure object describing the VisualTab (and the number and species of
	 * its VisualiserPanels)
	 */
	private Figure mFigure;

	// private Figure resetFigure;

	/*
	 * The JPanel on which the VisualPanels are layed
	 */
	private JPanel mJpMain;

	/*
	 * The index of the VisualiserTab within the Visualiser
	 */
	private int mSimID = 0;

	/*
	 * The number of the current rendered iteration
	 */
	// private int currentIteration = -1;

	/*
	 * The maximum number of iterations to be rendered
	 */
	private int mMaxIterations = 10;

	/*
	 * The GridLayout used for the visualiserTab
	 */
	private GridLayout mTabLayout;

	/*
	 * A boolean flag specifying if the VisualiserPanels should be synchronised
	 */
	// private boolean syncCDPs = true;

	/*
	 * The general zoomRatio of the VisualiserPanels
	 */
	private double mZoomRatio = 1;

	/*
	 * @param vis: IVisualiser reference
	 * 
	 * @param Figure f: the figure that will be rendered in the VisualiserTab
	 * 
	 * @param simID: the id of the simulation (of the Tab) within the
	 * VRootModel's SimulationRun list
	 */
	public VisualiserTab(IVisualiser vis, Figure figure, int simID) {
		mvis = vis;
		setLayout(new GridLayout(0, 1));
		if (figure == null) {
			mFigure = new Figure(new Species("auxin", "Auxin",
					Species.Type.CELL), 0, 0);
		} else
			mFigure = figure;
		initComponents();
		setFigure(mFigure);
		this.mSimID = simID;
	}

	public void setLoadedSpecies(Species[] species) {
		// TODO Auto-generated method stub
		mFigure.setSpecies(species);
		for (VisualiserPanel vp : mVisPanels) {
			vp.setLoadedSpecies(species);
		}
	}

	public int getSimID() {
		return mSimID;
	}

	private void initComponents() {
		mTabLayout = new GridLayout(0, 1);
		mJpMain = new JPanel(mTabLayout);
		mJpMain.setPreferredSize(new Dimension(750, 400));
		mJpMain.setBackground(Color.WHITE);

		mVisPanels = new ArrayList<VisualiserPanel>();

		int vpIndex = 0;
		CellDrawingPanel view = new CellDrawingPanel(vpIndex);
		view.setZoomRatio(mZoomRatio);
		CDPController controller = new CDPController(this,
				mvis.getTabHistDataProvider(), view);
		VisualiserPanel vpanel = new VisualiserPanel(vpIndex, mvis, view,
				controller, this);
		vpanel.setZoomValue(mZoomRatio);
		vpanel.setMaxIteration(mMaxIterations);

		mVisPanels.add(vpanel);
		mJpMain.add(vpanel);
		add(mJpMain);
	}

	/*
	 * Sets the Figure for the tab and its VisualiserPanels
	 */
	public void setFigure(final Figure figure) {
		final VisualiserTab thisTab = this;

		int panelNO = figure.getSelectedSpeciesCount();
		// reset the number of columns
		mTabLayout = new GridLayout(0, figure.getNoColumns());
		mJpMain.setLayout(mTabLayout);

		// normalise the number of VisualiserPanels
		if (panelNO > mVisPanels.size()) {
			for (int i = mVisPanels.size(); i < panelNO; i++) {
				CellDrawingPanel view = new CellDrawingPanel(i);
				// view.setZoomRatio(zoomRatio);
				CDPController controller = new CDPController(thisTab,
						mvis.getTabHistDataProvider(), view);
				VisualiserPanel vpanel = new VisualiserPanel(i, mvis, view,
						controller, thisTab);
				// vpanel.setZoomValue(zoomRatio);
				vpanel.setMaxIteration(mMaxIterations);
				mVisPanels.add(vpanel);
				mJpMain.add(vpanel);
			}
		} else if (panelNO < mVisPanels.size()) {
			for (int i = mVisPanels.size() - 1; i >= panelNO; i--) {
				VisualiserPanel vpanel = mVisPanels.get(i);
				mVisPanels.remove(i);
				mJpMain.remove(vpanel);
				vpanel = null;
			}
		}
		validate();
		// set the species for each panel
		for (int i = 0; i < figure.getSelectedSpeciesCount(); i++) {
			VisualiserPanel vp = mVisPanels.get(i);
			vp.setFigure(figure, figure.getSelectedSpeciesIndex()[i]);
			vp.setRecord(getRecord(), true, true);
		}
		// conclude
		mFigure = figure;
	}

	public void closeVisualiserPanel(int i) {
		for (VisualiserPanel panel : mVisPanels) {
			if (panel.getIndex() == i) {
				mVisPanels.remove(panel);
				break;
			}
		}
		// visPanels.remove(i);
		this.removeAll();
		if (mVisPanels.size() == 1)
			mTabLayout = new GridLayout(0, 1);
		else
			mTabLayout = new GridLayout(0, mFigure.getNoColumns());
		mJpMain = new JPanel(mTabLayout);
		for (VisualiserPanel panel : mVisPanels) {
			mJpMain.add(panel);
		}
		add(mJpMain);
		repaint();
		validate();
	}

	public Figure getFigure() {
		return mFigure;
	}

	public void setTitle(String title) {
		// TODO
	}

	/*
	 * 
	 */
	public void setRecord(Record record, boolean setControlBar, boolean initial) {
		mRecord = record;
		for (VisualiserPanel visPanel : mVisPanels)
			visPanel.setRecord(record, setControlBar, initial);
	}

	public Record getRecord() {
		return mRecord;
	}

	public void setMaxIterations(int maxIterations) {
		if (mVisPanels != null)
			this.mMaxIterations = maxIterations;
		for (VisualiserPanel visPanel : mVisPanels) {
			visPanel.setMaxIteration(maxIterations);
		}
	}

	/*
	 * Makes all CellDrawingPanels in the tab have the same zoom factor as the
	 * first CellDrawingPanel
	 */
	public void homogeniseZoomRatio() {
		// final double s = visPanels[0].getZoomRatio();
		final double s = mVisPanels.get(0).getZoomRatio();
		if (mVisPanels.size() > 1)
			for (int i = 1; i < mVisPanels.size(); i++) {
				// final VisualiserPanel vp = visPanels[i];
				final VisualiserPanel vp = mVisPanels.get(i);
				SwingUtilities.invokeLater(new Runnable() {
					public void run() {
						vp.setZoomRatio(s);
					}
				});
			}
		mZoomRatio = s;
	}

	/*
	 * Applies the same transform to all CellDrawingPanels
	 */
	public void applyTransform(int cdpID, double zoom, double tx, double ty,
			final boolean isChangedBySpinner) {
		final double tZoom = zoom;
		final double ttx = tx;
		final double tty = ty;
		// if (syncCDPs)
		for (VisualiserPanel vp : mVisPanels) {
			// final VisualiserPanel vp = visPanels.get(i);
			if (vp.isViewSynchronised() || vp.getIndex() == cdpID)
				vp.applyTransform(tZoom, ttx, tty, isChangedBySpinner);
		}
		// else {
		// final VisualiserPanel vp = visPanels.get(cdpID);
		// vp.applyTransform(tZoom, ttx, tty, isChangedBySpinner);
		// }

		mZoomRatio = mVisPanels.get(0).getZoomRatio();
	}

	public void setSpinnerValue() {
		for (int i = 0; i < mVisPanels.size(); i++) {
			final VisualiserPanel vp = mVisPanels.get(i);
			vp.setZoomValue(mZoomRatio);
		}
	}

	public void setFitZoomRatio() {
		for (int i = 0; i < mVisPanels.size(); i++) {
			final VisualiserPanel vp = mVisPanels.get(i);
			vp.setFitZoomRatio();
		}
	}

	public void fireCellSelected(Record currentRecord, Cell selectedCell) {
		for (VisualiserPanel c : mVisPanels) {
			c.getController().fireCellSelected(mSimID, currentRecord,
					selectedCell);
		}
		mvis.fireCellSelected(mSimID, currentRecord, selectedCell);
	}

	public void fireCellUnselected() {
		for (VisualiserPanel c : mVisPanels) {
			c.getController().fireCellUnselected(mSimID);
		}
		mvis.fireCellUnselected(mSimID);
	}

	/*
	 * 
	 */
	public void setRenderedIteration(int iteration) {
		setRenderedIteration(iteration, -1);
	}

	/*
	 * 
	 */
	public void setRenderedIteration(int iteration, int vpIndex) {
		final Record r = mvis.getTabHistDataProvider().getRecord(mSimID,
				iteration);

		// currentIteration = iteration;
		for (int i = 0; i < mVisPanels.size(); i++) {
			final VisualiserPanel vp = mVisPanels.get(i);
			if (vp.isViewSynchronised() || (i == vpIndex))
				// new Thread(new Runnable() {
				//
				// @Override
				// public void run() {
				vp.setRecord(r, true, false);
			// }
			// }).start();
		}
	}

	public void reset() {
		for (VisualiserPanel vp : mVisPanels)
			vp.reset();
	}
}
