import lxml.etree as ET

import numpy as np
#import matplotlib.pylab as plt
#from matplotlib.collections import PolyCollection, CircleCollection


from openalea.container import Topomesh, Quantity, ordered_pids
from openalea.celltissue import Tissue, topen, ConfigFormat, TissueDB, \
    ConfigItem
from geom import area
from openalea.tissueshape import face_surface_2D,edge_length


"""
TODO

1. Merge walls between internal cells - too many sub-nodes.
2.     
"""




# Routine to convert xml attributes to strings or numbers,
# as appropriate
def to_val(s):
    s=str(s)
    if s=='None':
        return None
    else:
        try:
            s=int(s)
        except:
            try:
                s=float(s)
            except:
                pass
            pass
        return s

def add_VS(db):   
    t = db.tissue()
    cfg = db.get_config('config')
    mesh = t.relation(cfg.mesh_id)
    pos = db.get_property('position')

    V = {}
    for cid in mesh.wisps(2):
        V[cid] = face_surface_2D(mesh, pos, cid)
   
    S = {}
    for wid in mesh.wisps(1):
        S[wid] = edge_length(mesh, pos, wid)
    
    db.set_description('wall', "wall corresponding to a given edge")
    db.set_property('V', V)
    db.set_description('V', "Volume of each cell")
    db.set_property('S', S)
    db.set_description('S', "Surface of each wall in m")

def add_graph(db):
    t = db.tissue()
    cfg = db.get_config('config')
    mesh = t.relation(cfg.mesh_id)
    pos = db.get_property('position')
    EDGE = t.add_type('EDGE')    
    graph_id = t.add_relation("graph",(cfg.CELL, EDGE))
    graph = t.relation(graph_id)
    cfg.add_item(ConfigItem('graph_id', graph_id))
    cfg.add_item(ConfigItem('EDGE', EDGE))

    wall = {}

    for wid in mesh.wisps(1) :
	if mesh.nb_regions(1,wid) == 2 :
		cid1,cid2 = mesh.regions(1,wid)
		eid1 = graph.add_edge(cid1,cid2)
		wall[eid1] = wid
		eid2 = graph.add_edge(cid2,cid1)
		wall[eid2] = wid

    db.set_property('wall', wall)
    db.set_description('wall', "wall corresponding to a given edge")


def load_xml(filename):

    wall_data={}
    cell_data=[]

    p_map={}

    t=Tissue()
    POINT = t.add_type('POINT')
    WALL = t.add_type('WALL')
    CELL = t.add_type('CELL')
    mesh_id = t.add_relation('mesh', (POINT,WALL,CELL) )
    mesh = t.relation(mesh_id)


    tree=ET.parse(filename)
    root=tree.getroot()
    cells=tree.find('cells')
    cell_list = []
    cell_group_list = []
    old_cell_id_list = []
    truncated_list = []
    for c in cells.iter('cell'):
        walls=c.find('walls')
        cell_walls=[]
        for w in walls.iter('wall'):
            cell_walls.append(int(w.get('id')))
        cell_data.append(cell_walls)
        cell_group_list.append(int(c.get('group')))
        old_cell_id_list.append(int(c.get('id')))
        truncated_list.append(c.get('truncated') == 'True')
        
  
    walls=tree.find('walls')
    wc = 0
    for w in walls.iter('wall'):
        pts=[]
        for point in w.find('points').iter('point'):
            pts.append((float(point.get('x')), float(point.get('y'))))
#        wall_data[int(w.get('id'))]=pts#[pts[0], pts[-1]]
        wall_data[int(w.get('id'))]=[pts[0], pts[-1]]
#        print [pts[0], pts[-1]]
        wc+=1


    pt_map={}
    wall_map={}
    pos={}
    for i, w in wall_data.iteritems():
        wall_map[i]=[]
        for pp in zip(w[0:-1], w[1:]):
            wid=mesh.add_wisp(1)
            for p in pp:
                if p not in pt_map:
                    pid=mesh.add_wisp(0)
                    pos[pid]=p
                    pt_map[p]=pid
                mesh.link(1, wid, pt_map[p])
            wall_map[i].append(wid)

    cid_list=[]
    for c in cell_data:
        cid=mesh.add_wisp(2)
        cid_list.append(cid)
        for w in c:
            for wid in wall_map.get(w,[]):
                mesh.link(2, cid, wid)

  
    

    cell_type = dict((cid, cell_group_list[i]) 
                      for i, cid in enumerate(cid_list))

   # cell_types =dict((i,str(i)) for i in set(cell_group_list))
    cell_types = { 2: 'epidermis',
                   4: 'cortex',
                   3: 'endodermis',
                   5: 'stele',
                   6: 'LRC1',
                   7: 'LRC2',
                   8: 'LRC3',
                   17: 'QC',
                   10: 'initials',
                   11: 'S1',
                   12: 'S2',
                   13: 'S3',
                   14: 'S4',
                   15: 'S5' }

                   

    old_cell_id = dict((cid, old_cell_id_list[i]) 
                      for i, cid in enumerate(cid_list))

    border_dict = dict((cid_list[i], 1) 
                  for i,t in enumerate(truncated_list) if t)
   
  
    pos = Quantity(pos,'pix','tuple',"position of vertices")
    cfg = ConfigFormat(locals())
    cfg.add_section("tissue descr")
    cfg.add('POINT')
    cfg.add('WALL')
    cfg.add('CELL')
    cfg.add('mesh_id')
    cfg.add_section("prop types")
    cfg.add('cell_types')
    cfg.add_item(ConfigItem('border', {1:'truncated'}))
    #writing

    db=TissueDB()
    db.set_config('config', cfg.config())
    db.set_tissue(t)
    db.set_property('position', pos)
    db.set_description('position', "position")
    db.set_property('old_cell_id', old_cell_id)
    db.set_description('old_cell_id', "original cell id numbers in xml file")
    db.set_property('cell_type', cell_type)
    db.set_description('cell_type', "type of each cell")
    db.set_property('border', border_dict)
    db.set_description('border',"")

    add_graph(db)
    add_VS(db)
    
    return db

if __name__=='__main__':
    import sys
    if len(sys.argv)<3:
        db = load_xml('geometries.xml')
        db.write('snakes.zip')
    else:
        db = load_xml(sys.argv[1])
        db.write(sys.argv[2])
    

