
"""
Module to plot tissue using matplotlib rather than OpenAlea
"""

import os, errno

class OutputDirectory(object):
    def __init__(self, output_folder, base):
        # http://stackoverflow.com/questions/600268/mkdir-p-functionality-in-python
        try:
            os.makedirs(output_folder)
        except OSError as exc: 
            if exc.errno == errno.EEXIST:
                pass
            else: raise

        self.output_folder = output_folder
        self.count = 0
        self.file = None
        self.base = base
        self.header = True

    def start_record(self):
        self.file = open(self.output_folder+'/'+self.base+'%05d.dat'%self.count,'w')

    def end_record(self):
        self.file.close()
        self.count += 1


class OutputFile(object):
    def __init__(self, outfile):
        self.file = outfile
        self.count = 0
        self.header = True

    def start_record(self):
        pass

    def end_record(self):
#        self.file.write('\n')
#        self.file.flush()
        self.count += 1
        self.header = False
        


def json_line(data):
    return json.dumps(data)

def csv_line(data):
    return ','.join(map(str, data))+'\n'

class CellSetOutput(object):
    def __init__(self, db, prop_names, writer):
        self.db = db
        self.writer = writer
        self.prop_names = prop_names

    def step(self):
        old_cell_ids = self.db.get_property('old_cell_id')
        cellset_ids = sorted((v,k) for k,v in old_cell_ids.iteritems())
        self.writer.start_record()
        if self.writer.header:
            self.writer.file.write(csv_line([i[0] for i in cellset_ids]))
        for prop_name in self.prop_names:
            prop = self.db.get_property(prop_name)
            self.writer.file.write(csv_line([prop[i[1]] for i in cellset_ids]))
        self.writer.end_record()

class DictOutput(object):
    def __init__(self, db, prop_names, writer):
        self.db = db
        self.writer = writer

    def step(self):
        self.writer.start_record()
        for prop_name in self.prop_names:
            prop = db.get_property(prop_name)
            self.writer.file.write(json_line(prop))
        self.writer.end_record()


from db_utilities import get_tissue_maps
        
def get_property_array(db, name):
    if 'old_cell_id' in self.db.properties():
        cid_to_cidx = self.db.get_property('old_cell_id')
    else:
        cid_to_cidx = get_tissue_maps(db)['cell']
    cidx_to_cid = sorted((v,k) for k,v in old_cell_ids.iteritems())
        
