"""
Initialize parameter values for simulation components
"""

from db_utilities import get_parameters, set_parameters


def setSimParameters(db):
    initial_values = { 'auxin': 0.0 }
    # Initial values for each property

    timestep = 0.002 
    # Initial global simulation timestep
#    print db.get_config('config').cell_types
#    print db.get_config('config').border

			#key    #list of touples (predicate -> value)
#    fixed_settings = { 'auxin': [('border[cid]==6 and cell_type[cid] in [4]', 2)] } #evaluate boundary condition
    fixed_settings = { 'auxin': [('border[cid]==1 and cell_type[cid] in [5]', 1), ('border[cid]==1 and cell_type[cid] in [2,3,4]', 0)] } #evaluate boundary condition
    set_parameters(db, 'initial_values', initial_values)
    set_parameters(db, 'timestep', timestep)
    set_parameters(db, 'fixed', fixed_settings)

def setViewParameters(db):
    from vplants.plantgl.ext.color import JetMap, GrayMap, GreenMap
    cmap = JetMap(0., 13., outside_values=True)
    p = { 'view_names': ['cell_types'], 'cmaps': [cmap],
          'edge_cmaps': [JetMap(0.0, 3.0, outside_values=True)],
          'edge_names': ['PIN'] }
    set_parameters(db, 'viewer', p)

def writeParameters(db, f):
    # Write parameters of db to filename
    params = db.get_property('parameters')
    for param_name, prop in params.iteritems():
        f.write(param_name + '\n')
        if isinstance(prop, dict):
            for key, value in prop.iteritems():
                f.write('    ' + str(key) + ' : ' + str(value) + '\n')
        else:
            f.write('    '+ str(prop) + '\n')
        f.write('\n')


import xml.dom.minidom

xmlParamSetExample = """\
<parameterset>
<title>Demo slideshow</title>
<parameter>
   <title>A3</title>
   <value>2</value>
   <type>int</type>
   <model>Auxin</model>
</parameter>

<parameter>
   <title>A4</title>
   <value>55</value>
   <type>float</type>
   <model>Auxin</model>
</parameter>
</parameterset>
"""

def writeDBtoXMLParameterset(db, outputFile = 'params.xml'):
    # Write parameters of db to filename
    params = db.get_property('parameters')
    dbPars = {}
    for param_name, prop in params.iteritems():
        dbPar = {'name':param_name}
        if isinstance(prop, dict):
            for key, value in prop.iteritems():
                t = {'name':param_name + "." + str(key)}
                t['value'] = str(value)
		t['model'] = param_name
                dbPars[t['name']] = t
        else:
            dbPar['value'] = str(prop)
            dbPars[param_name] = dbPar
    writeXMLParameterset(dbPars, outputFile)    

def writeXMLParameterset(dbPars, outputFile = 'params.xml'):
    f=open(outputFile,'w')
    f.write('<parameterset>\n')
    #TODO something about the title
    for parTitle, dbPar in dbPars.iteritems():
        f.write('<parameter>\n')
        f.write("    <title> %s </title>\n" % dbPar['name'])
        f.write("    <value> %s </value>\n" % dbPar['value'])
        if 'type' in dbPar:
            f.write("    <type> %s </type>\n" % dbPar['type'])
        if 'model' in dbPar:
            f.write("    <model> %s </model>\n" % dbPar['model'])
        f.write('</parameter>\n')
    f.write('</parameterset>\n')
    f.close()

def readXMLParameterset(xmlFile):
    f = open(xmlFile,'r')
    return parseXMLParameterset(f.read())

def parseXMLParameterset(xmlString):
    """
    Converts an XML string into a dictionary of parameters 
    (each key in this dictinary is the parameter name, 
    whereas each value is a dictionary defining the <name>, <value>, <type> and <model> of that parameter).

    Returns the resulting parameters dictionary.
    """
    xmlString = xml.dom.minidom.parseString(xmlString)
    #print "<parameterset>"
    handleParametersetTitle(xmlString.getElementsByTagName("title")[0])
    parameters = xmlString.getElementsByTagName("parameter")
    #handleToc(parameters)
    return handleParameters(parameters)
    #print "</parameterset>"

def handleParameters(parameters, dbPars={}):
    """
    Returns a dictionary of parameters, as parsed from the parameters string
    """
    for parameter in parameters:
        handleParameter(parameter, dbPars)
    return dbPars

def handleParameter(parameter, dbPars):    
    parTitle = handleParameterTitle(parameter.getElementsByTagName("title")[0])    
    parType = handleValue(parameter.getElementsByTagName("type")[0])
    parVal = handleValue(parameter.getElementsByTagName("value")[0])
    parMod = handleValue(parameter.getElementsByTagName("model")[0])
    par = {'name':parTitle, 'type':parType, 'value':parVal, 'model':parMod}
    dbPars[parTitle] = par

def handleParametersetTitle(title):
    parTitle = getText(title.childNodes) 
    #print "<title>%s</title>" % parTitle
    return parTitle

def handleParameterTitle(title):
    parTitle = getText(title.childNodes)
    #print "<par>%s</par>" % parTitle
    return parTitle

def handleValue(point):
    parValue = getText(point.childNodes)
    #print "<val>%s</val>" % parValue
    return parValue

def handleToc(parameters):
    for parameter in parameters:
        title = parameter.getElementsByTagName("title")[0]
        #print "<toc>%s</toc>" % getText(title.childNodes)

def getText(nodelist):
    rc = []
    for node in nodelist:
        if node.nodeType == node.TEXT_NODE:
            rc.append(node.data)
    #print ''.join(rc)
    #exit(1)
    return ''.join(rc)

