# Program by Leah, 13th July 2012
# Runs auxin transport (IAAH diffusion across membranes, anionic active transport via PIN and AUX1) and cell-wall diffusion (assuming a auxin concentration at each vertex, and then taking the limit in which the volume of these compartments is zero), together with reduced DII-VENUS model (a redimensionalised version of equation (2.11) in DII gravitropism paper's Supplementary material)
# Simulates auxin concentration in every cell and every wall segment. DII concentration in every cell.
# Compares simulations with DII-VENUS intensity measurements.

import os, sys
import cPickle as pickle

from db_utilities import get_mesh, get_graph, get_parameters, set_parameters, get_wall_decomp
from openalea.container import ordered_pids

from numpy import *
from pylab import *

import matplotlib
from matplotlib.patches import Polygon
from matplotlib.collections import PatchCollection
import matplotlib.pyplot as plt

def plotPINs(db,fname):       
    graph = get_graph(db)  
    mesh = get_mesh(db)    
    PIN = db.get_property('PIN')
    position = db.get_property('position')
    wall_decomposition = get_wall_decomp(db)
  

    # Calculate the co-ordinates of each cell.
    cell_to_coordinates = CellToCoordinates_plot(mesh,position)  # This is defined at the end of this script.

    # Calculate centroid and size of geometry (xmin, xmax, ymin, ymax).
    centroid={}
    xmin=10000
    xmax=0
    ymin=10000
    ymax=0
    for cid in mesh.wisps(2):
	xc=0
	yc=0	
	coords=cell_to_coordinates[cid]
	for j in range(0, size(coords)/2-1):
	    xc=xc+coords[j][0]
	    yc=yc+coords[j][1]		
	    centroid[cid]=[xc/(size(coords)/2-1),yc/(size(coords)/2-1)]
	    if xmax<coords[j][0]:
		xmax=coords[j][0]
	    if xmin>coords[j][0]:
		xmin=coords[j][0]
	    if ymax<coords[j][1]:
		ymax=coords[j][1]
	    if ymin>coords[j][1]:
		ymin=coords[j][1]
 
    fig=figure(figsize=(25,7))
    ax=fig.add_subplot(111,aspect='equal')
    ax.set_yticks([])
    ax.set_xticks([])
    box(on=None)	
    xlim(xmin,xmax )    # set the xlim to xmin, xmax
    ylim(ymin, ymax )    # set the ylim to ymin, ymax

    for cid in mesh.wisps(2):
	c = centroid[cid]

	for wid in mesh.borders(2,cid):  # mesh.borders(2,cid) lists the ids of walls surrounding cell cellid.
            pid0, pid1 = mesh.borders(1, wid)  # Possibly could be simplified using border_to_coordinates
            gca().add_line(Line2D((position[pid0][0],position[pid1][0]),(position[pid0][1],position[pid1][1]), lw=1))

	    for eid in wall_decomposition.get(wid, []):
	        if graph.source(eid) == cid and PIN[eid]==1:
		    pid0, pid1 = mesh.borders(1,wid)
		    pos0 = [position[pid0][0]*0.85+c[0]*0.15,position[pid0][1]*0.85+c[1]*0.15]
		    pos1 = [position[pid1][0]*0.85+c[0]*0.15,position[pid1][1]*0.85+c[1]*0.15]				
		    gca().add_line(Line2D((pos0[0],pos1[0]),(pos0[1],pos1[1]), color='r', lw=2))
			
    figtext(0.7, 0.85, 'PIN positions',size='35')
    outputFolder='./'
    savefig(outputFolder + fname +'.png')
    clf()

# AUX1 figure
def plotAUX1(db,fname):       
    graph = get_graph(db)  
    mesh = get_mesh(db)    
    AUX1 = db.get_property('AUX')
    position = db.get_property('position')
    wall_decomposition = get_wall_decomp(db)
  

    # Calculate the co-ordinates of each cell.
    cell_to_coordinates = CellToCoordinates_plot(mesh,position)  # This is defined at the end of this script.

    # Calculate centroid and size of geometry (xmin, xmax, ymin, ymax).
    centroid={}
    xmin=10000
    xmax=0
    ymin=10000
    ymax=0
    for cid in mesh.wisps(2):
	xc=0
	yc=0	
	coords=cell_to_coordinates[cid]
	for j in range(0, size(coords)/2-1):
	    xc=xc+coords[j][0]
	    yc=yc+coords[j][1]		
	    centroid[cid]=[xc/(size(coords)/2-1),yc/(size(coords)/2-1)]
	    if xmax<coords[j][0]:
		xmax=coords[j][0]
	    if xmin>coords[j][0]:
		xmin=coords[j][0]
	    if ymax<coords[j][1]:
		ymax=coords[j][1]
	    if ymin>coords[j][1]:
		ymin=coords[j][1]
 
    fig=figure(figsize=(25,7))
    ax=fig.add_subplot(111,aspect='equal')
    ax.set_yticks([])
    ax.set_xticks([])
    box(on=None)	
    xlim(xmin,xmax )    # set the xlim to xmin, xmax
    ylim(ymin, ymax )    # set the ylim to ymin, ymax

    for cid in mesh.wisps(2):
	c = centroid[cid]

	for wid in mesh.borders(2,cid):  # mesh.borders(2,cid) lists the ids of walls surrounding cell cellid.
	    pid0, pid1 = mesh.borders(1, wid)  # Possibly could be simplified using border_to_coordinates
	    gca().add_line(Line2D((position[pid0][0],position[pid1][0]),(position[pid0][1],position[pid1][1]), lw=1))

	    for eid in wall_decomposition.get(wid, []):
		if graph.source(eid) == cid and AUX1[eid]>0:		    
		    if AUX1[eid]==1:
		        pid0, pid1 = mesh.borders(1,wid)
		        pos0 = [position[pid0][0]*0.95+c[0]*0.05,position[pid0][1]*0.95+c[1]*0.05]
		        pos1 = [position[pid1][0]*0.95+c[0]*0.05,position[pid1][1]*0.95+c[1]*0.05]				
			gca().add_line(Line2D((pos0[0],pos1[0]),(pos0[1],pos1[1]), color='g', lw=4))

    figtext(0.7, 0.85, 'AUX1 positions',size='35')
    outputFolder = './'
    savefig(outputFolder + fname +'.png')
    clf()

def CellToCoordinates_plot(mesh,position):
    cell_to_walls = dict((cid,tuple(mesh.borders(2,cid))) for cid in mesh.wisps(2))
    border_to_coordinates = dict((cid,\
                        [[position[tuple(mesh.borders(1,cid))[0]][0],\
                          position[tuple(mesh.borders(1,cid))[0]][1]],\
                         [position[tuple(mesh.borders(1,cid))[1]][0],\
                          position[tuple(mesh.borders(1,cid))[1]][1]]]\
                        ) for cid in mesh.wisps(1))

    ptc=[]
    for ncid,cid in enumerate(mesh.wisps(2)):
        ptc.append([])
        patches=[]
  
        for i in range(len(cell_to_walls[cid])):
            patches.append(border_to_coordinates[cell_to_walls[cid][i]])

        ptc[ncid]=[]
        ptc[ncid].append(patches[0][0])
        ptc[ncid].append(patches[0][1])
        patches.remove(patches[0])
        i=0
        while i <len(patches):
            if patches[i].count(ptc[ncid][-1])==1:
                id1=patches[i].index(ptc[ncid][-1])
		id2=abs(-1+id1)
                ptc[ncid].append(patches[i][id2])
                patches.remove(patches[i])
                i=-1
            i=i+1

    cell_to_coordinates = dict((cid,ptc[ncid]) for ncid,cid in enumerate(mesh.wisps(2)))
    return cell_to_coordinates 

