
from simulation import Simulation

from genenetwork import IntegrationException
# http://stackoverflow.com/questions/375427/non-blocking-read-on-a-subprocess-pipe-in-python

import sys
from threading  import Thread
try:
    from Queue import Queue, Empty
except ImportError:
    from queue import Queue, Empty 

ON_POSIX = 'posix' in sys.builtin_module_names

def enqueue_input(out, queue):
    for line in iter(out.readline, b''):
        queue.put(line)
    out.close()

import json

class FileConnection(object):
    def __init__(self, infile, outfile):
        self.infile = infile
        self.inqueue = Queue()
        self.inthread =  Thread(target=enqueue_input, 
                                args=(infile, self.inqueue))
        self.inthread.daemon = True
        self.inthread.start()
        self.outfile = outfile

    def send_message(self, m):
#        self.sbuffer += m+'\n'
        if type(m) is not str:
            self.outfile.write(json.dumps(m)+'\n')
        else:
            self.outfile.write(m+'\n')
        self.outfile.flush()

    def read_message(self):
        try:
            return self.inqueue.get(True, 1).strip()
        except Empty:
            return None

    def message_pending(self):
        return not self.inqueue.empty()







import cPickle
def load_state(filename):
    return cPickle.load(open(filename))

class SimDriver(object):
    def __init__(self, state_file=None, conn=None):
        if state_file:
            self.sim=load_state(state_file)
        else:
            self.sim = Simulation()
        if conn:
            self.conn = conn
        else:
            self.conn = FileConnection(sys.stdin, sys.stdout)
        self.state = 0

    def run(self):
        while True:
            if not self.conn.inthread.is_alive():
                break
            m = self.conn.read_message()
            if m:
                try:
                    reply=eval(m,{'s':self.sim})
                    if reply is not None:
                        self.conn.send_message(reply)
                    else:
                        self.conn.send_message("NONE")
                except Exception as e:
                    print e
                    self.conn.send_message("EXCEPTION")

if len(sys.argv)>1:
    sd = SimDriver(sys.argv[1])
else:
    sd = SimDriver()
sd.run()
